@@ -1,3 +1,36 @@
+/*
+
+  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+
+  All rights reserved.
+
+  Redistribution and use in source and binary forms, with or without
+  modification, are permitted provided that the following conditions
+  are met:
+
+  1. Redistributions of source code must retain the above copyright
+     notice, this list of conditions and the following disclaimer.
+  2. Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+  3. Neither the name of authors nor the names of its contributors
+     may be used to endorse or promote products derived from this software
+     without specific prior written permission.
+
+  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+  SUCH DAMAGE.
+
+*/
+
 /*
  * gtk+-immodule
  */
@@ -119,7 +152,7 @@ im_uim_commit_string(void *ptr, char *str)
 static void
 clear_cb(void *ptr)
 {
-  IMUIMContext *uic = ptr;
+  IMUIMContext *uic = (IMUIMContext *)ptr;
   int i;
   for (i = 0; i < uic->nr_psegs; i++) {
     free(uic->pseg[i].str);
@@ -132,10 +165,11 @@ clear_cb(void *ptr)
 static void
 pushback_cb(void *ptr, int attr, char *str)
 {
-  IMUIMContext *uic = ptr;
+  IMUIMContext *uic = (IMUIMContext *)ptr;
   g_return_if_fail(str);
   
-  if(!strcmp(str,"") && !(attr & (UPeAttr_Cursor | UPeAttr_Separator)))
+  if(!strcmp(str,"")
+     && !(attr & (UPreeditAttr_Cursor | UPreeditAttr_Separator)))
     return;
 
   uic->pseg = realloc(uic->pseg,
@@ -149,8 +183,8 @@ pushback_cb(void *ptr, int attr, char *str)
 static void
 update_cb(void *ptr)
 {
-  IMUIMContext *uic = ptr;
-  g_return_if_fail(ptr);
+  IMUIMContext *uic = (IMUIMContext *)ptr;
+  g_return_if_fail(uic);
   g_signal_emit_by_name(uic, "preedit_changed");
 }
 
@@ -184,6 +218,10 @@ convert_keyval(int key)
   case GDK_Alt_R: return UKey_Alt_key;
   case GDK_Meta_L: return UKey_Meta_key;
   case GDK_Meta_R: return UKey_Meta_key;
+  case GDK_Super_L: return UKey_Super_key;
+  case GDK_Super_R: return UKey_Super_key;
+  case GDK_Hyper_L: return UKey_Hyper_key;
+  case GDK_Hyper_R: return UKey_Hyper_key;
   }
 
   if(key >= GDK_F1 && key <= GDK_F35) {
@@ -203,13 +241,19 @@ convert_modifier(int mod)
 {
   int rv = 0;
   if (mod & GDK_SHIFT_MASK) {
-    rv |= UKey_Shift;
+    rv |= UMod_Shift;
   }
   if (mod & GDK_CONTROL_MASK) {
-    rv |= UKey_Control;
+    rv |= UMod_Control;
   }
   if (mod & GDK_MOD1_MASK) {
-    rv |= UKey_Alt;
+    rv |= UMod_Alt;
+  }
+  if (mod & GDK_MOD3_MASK) {  /* assuming mod3 */
+    rv |= UMod_Super;
+  }
+  if (mod & GDK_MOD4_MASK) {  /* assuming mod4 */
+    rv |= UMod_Hyper;
   }
   return rv;
 }
@@ -227,7 +271,8 @@ filter_keypress(GtkIMContext *ic,
 }
 
 static gboolean
-get_user_defined_color(PangoColor *color, const gchar *uim_symbol) {
+get_user_defined_color(PangoColor *color, const gchar *uim_symbol)
+{
   gboolean parsed = FALSE;
   gchar *literal = uim_symbol_value_str(uim_symbol);
   if(literal != NULL && literal[0] != '\0') {
@@ -244,7 +289,7 @@ get_preedit_segment(struct preedit_segment *ps,
 {
   PangoAttribute *attr;
   const gchar *segment_str = ps->str;
-  if ((ps->attr & UPeAttr_Separator) && !strcmp(segment_str, "")) {
+  if ((ps->attr & UPreeditAttr_Separator) && !strcmp(segment_str, "")) {
     segment_str = DEFAULT_SEPARATOR_STR;
   }
   if (attrs) {
@@ -252,15 +297,15 @@ get_preedit_segment(struct preedit_segment *ps,
     int begin, end;
     begin = strlen(str);
     end = begin + strlen(segment_str);
-    if (ps->attr & UPeAttr_UnderLine) {
+    if (ps->attr & UPreeditAttr_UnderLine) {
       attr = pango_attr_underline_new(PANGO_UNDERLINE_SINGLE);
       attr->start_index = begin;
       attr->end_index = end;
       pango_attr_list_change(attrs, attr);
     }
-    if (ps->attr & UPeAttr_Separator) {
+    if (ps->attr & UPreeditAttr_Separator) {
       const gchar *separator_fg_symbol, *separator_bg_symbol;
-      if (ps->attr & UPeAttr_Reverse) {
+      if (ps->attr & UPreeditAttr_Reverse) {
 	separator_fg_symbol = "reversed-separator-foreground";
 	separator_bg_symbol = "reversed-separator-background";
       } else {
@@ -282,7 +327,7 @@ get_preedit_segment(struct preedit_segment *ps,
 	attr->end_index = end;
 	pango_attr_list_change (attrs, attr);
       }
-    } else if (ps->attr & UPeAttr_Reverse) {
+    } else if (ps->attr & UPreeditAttr_Reverse) {
       if (get_user_defined_color(&color, "reversed-preedit-foreground")
 	  || pango_color_parse(&color, "#fff"))
       {
@@ -322,7 +367,7 @@ im_uim_get_preedit_string(GtkIMContext *ic, gchar **str,
   }
   tmp = g_strdup("");
   for (i = 0; i < uic->nr_psegs; i++) {
-    if (uic->pseg[i].attr & UPeAttr_Cursor) {
+    if (uic->pseg[i].attr & UPreeditAttr_Cursor) {
       pos = g_utf8_strlen(tmp,-1);
     }
     if (attrs) {
@@ -393,24 +438,23 @@ focus_in(GtkIMContext *ic)
   check_helper_connection();
 
   uim_prop_list_update(uic->uc);
+  uim_prop_label_update(uic->uc);
 
   for (cc = context_list.next; cc != &context_list; cc = cc->next) {
     if (cc != uic) {
-      if (uic->cwin.top_win) {
-	gtk_widget_hide(uic->cwin.top_win);
+      if (cc->cwin.top_win) {
+	gtk_widget_hide(cc->cwin.top_win);
       }
     }
   }
-
-  if (uic->win && uic->toplevel) {
-    gdk_window_add_filter(uic->toplevel,
-			  toplevel_window_candidate_cb,
-			  uic);
+  
+  if(uic->cwin.top_win && uic->cwin.is_active){
+    gtk_widget_show(uic->cwin.top_win);
   }
   
   uim_helper_client_focus_in(uic->uc);
-
 }
+
 static void
 focus_out(GtkIMContext *ic)
 {
@@ -421,9 +465,6 @@ focus_out(GtkIMContext *ic)
 
   uim_helper_client_focus_out(uic->uc);
 
-  if (uic->win) {
-    gdk_window_remove_filter(uic->toplevel, toplevel_window_candidate_cb, uic);
-  }
   if(uic->cwin.top_win){
     gtk_widget_hide(uic->cwin.top_win);
   }
@@ -506,7 +547,7 @@ show_preedit(GtkIMContext *ic, GtkWidget *preedit_label)
 static GdkFilterReturn
 toplevel_window_candidate_cb (GdkXEvent *xevent, GdkEvent *ev, gpointer data)
 {
-  IMUIMContext *uic = focused_context;
+  IMUIMContext *uic = data;
 
   if(!uic)
     return GDK_FILTER_CONTINUE;
@@ -522,23 +563,10 @@ static void
 set_client_window(GtkIMContext *ic, GdkWindow *w)
 {
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
-  GdkWindow *toplevel = NULL;
 
   if(w) {
-    toplevel = gdk_window_get_toplevel(w);
+    g_object_ref(w);
   }
-
-  if(uic->toplevel != toplevel) {
-    
-    if(toplevel) {
-      g_object_ref(toplevel);
-    }
-    if(uic->toplevel) {
-      g_object_unref(uic->toplevel);
-    }
-    uic->toplevel = toplevel;
-  }
-
   uic->win = w;
 }
 
@@ -564,7 +592,7 @@ im_uim_finalize(GObject *obj)
   uic->menu = NULL;
 
   uim_release_context(uic->uc);
-
+  
   g_signal_handlers_disconnect_by_func(uic->slave,
 				       im_uim_commit_cb,
 				       uic);
@@ -622,9 +650,10 @@ select_candidate_cb(GtkWidget *clist,
 		    gpointer data)
 {
   IMUIMContext *uic = IM_UIM_CONTEXT(data);
+
   if (uic->cwin.select_count == 0) {
     int idx = row + uic->cwin.layout_begin;
-    
+  
     if( idx >= uic->cwin.nr_candidates ) {
       gtk_clist_select_row(GTK_CLIST(uic->cwin.clist),
 			   uic->cwin.candidate_index - uic->cwin.layout_begin,
@@ -635,7 +664,7 @@ select_candidate_cb(GtkWidget *clist,
     uim_set_candidate_index(uic->uc, idx);
     uic->cwin.candidate_index = idx;
   } else {
-    uic->cwin.select_count --;
+    uic->cwin.select_count = 0;
   }
 }
 
@@ -689,6 +718,7 @@ init_candidate_win(IMUIMContext *uic)
   gtk_widget_show(cwin->clist);
   gtk_widget_show(numlabel);
   gtk_widget_show(vbox);
+  
 }
 
 static int
@@ -744,7 +774,7 @@ calc_candidate_window_size(struct candidate_window cwin, int *width, int *height
   if(*width < 80)
     *width = 80;
 
-  gtk_window_resize(cwin.top_win, *width, *height);
+  gtk_window_resize(GTK_WINDOW(cwin.top_win), *width, *height);
 }
 
 static void
@@ -802,7 +832,7 @@ layout_candidate(IMUIMContext *uic)
     }
   }
   
-  uic->cwin.select_count ++;
+  uic->cwin.select_count = 1;
 
   if(uic->cwin.candidate_index >= 0) {
     gtk_clist_select_row(GTK_CLIST(uic->cwin.clist),
@@ -832,8 +862,8 @@ update_candidate(IMUIMContext *uic)
   for (i = 0; i < NR_CANDIDATES && i < uic->cwin.display_limit; i++) {
     if (i < uic->cwin.nr_candidates - uic->cwin.layout_begin) {
       cand = uim_get_candidate(uic->uc, uic->cwin.layout_begin + i, i);
-      buf[0] = uim_candidate_get_heading_label(cand);
-      buf[1] = uim_candidate_get_cand_str(cand);
+      buf[0] = (char *) uim_candidate_get_heading_label(cand);
+      buf[1] = (char *) uim_candidate_get_cand_str(cand);
       gtk_clist_append(GTK_CLIST(uic->cwin.clist), buf);
       uim_candidate_free(cand);
     } else {
@@ -846,9 +876,9 @@ update_candidate(IMUIMContext *uic)
 }
 
 static void
-cand_begin_cb(void *ptr, int nr, int display_limit)
+cand_activate_cb(void *ptr, int nr, int display_limit)
 {
-  IMUIMContext *uic = ptr;
+  IMUIMContext *uic = (IMUIMContext *)ptr;
   init_candidate_win(uic);
   uic->cwin.is_active = 1;
   uic->cwin.candidate_index = -1;
@@ -857,6 +887,14 @@ cand_begin_cb(void *ptr, int nr, int display_limit)
   uic->cwin.layout_begin = 0;
   layout_candidate(uic);
   gtk_widget_show(uic->cwin.top_win);
+
+  if (uic->win) {
+    GdkWindow *toplevel;
+    toplevel = gdk_window_get_toplevel(uic->win);
+    gdk_window_add_filter(toplevel,
+			  toplevel_window_candidate_cb,
+			  uic);
+  }
 }
 
 
@@ -865,7 +903,7 @@ cand_begin_cb(void *ptr, int nr, int display_limit)
 static void
 cand_select_cb(void *ptr, int index)
 {
-  IMUIMContext *uic = ptr;
+  IMUIMContext *uic = (IMUIMContext *)ptr;
   uic->cwin.candidate_index = index;
   layout_candidate(uic);
 }
@@ -873,7 +911,7 @@ cand_select_cb(void *ptr, int index)
 static void
 cand_shift_page_cb(void *ptr, int direction)
 {
-  IMUIMContext *uic = ptr;
+  IMUIMContext *uic = (IMUIMContext *)ptr;
 
   if(direction){
     if(uic->cwin.nr_candidates < uic->cwin.candidate_index + uic->cwin.display_limit )
@@ -892,12 +930,19 @@ cand_shift_page_cb(void *ptr, int direction)
 }
 
 static void
-cand_end_cb(void *ptr)
+cand_deactivate_cb(void *ptr)
 {
-  IMUIMContext *uic = ptr;
+  IMUIMContext *uic = (IMUIMContext *)ptr;
   if(uic->cwin.top_win) {
     gtk_widget_hide(uic->cwin.top_win);
   }
+
+  if (uic->win) {
+    GdkWindow *toplevel;
+    toplevel = gdk_window_get_toplevel(uic->win);
+    gdk_window_remove_filter(toplevel, toplevel_window_candidate_cb, uic);
+  }
+
   /*  uic->nr_psegs = 0; */
   uic->cwin.is_active = 0;
 }
@@ -944,10 +989,10 @@ im_module_create (const gchar *context_id)
   uim_set_prop_label_update_cb(uic->uc,
 			      update_prop_label_cb);
   uim_set_candidate_selector_cb(uic->uc,
-				cand_begin_cb,
+				cand_activate_cb,
 				cand_select_cb,
 				cand_shift_page_cb,
-				cand_end_cb);
+				cand_deactivate_cb);
   
   uim_prop_list_update(uic->uc);
 
@@ -1047,11 +1092,11 @@ im_uim_helper_disconnect_cb(void)
   gdk_input_remove(read_tag);
 }
 
-/* XXX:This is not recommended way!! */
+/* XXX:This is not a recommended way!! */
 gboolean
 uim_key_snoop(GtkWidget *grab_widget, GdkEventKey *key, gpointer data)
 {
-  if(focused_context){
+  if (focused_context) {
     int rv;
     int kv = convert_keyval(key->keyval);
     int mod = convert_modifier(key->state);
