@@ -1,4 +1,34 @@
-;;
+;;; anthy.scm: Anthy for uim.
+;;;
+;;; Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+;;;
+;;; All rights reserved.
+;;;
+;;; Redistribution and use in source and binary forms, with or without
+;;; modification, are permitted provided that the following conditions
+;;; are met:
+;;; 1. Redistributions of source code must retain the above copyright
+;;;    notice, this list of conditions and the following disclaimer.
+;;; 2. Redistributions in binary form must reproduce the above copyright
+;;;    notice, this list of conditions and the following disclaimer in the
+;;;    documentation and/or other materials provided with the distribution.
+;;; 3. Neither the name of authors nor the names of its contributors
+;;;    may be used to endorse or promote products derived from this software
+;;;    without specific prior written permission.
+;;;
+;;; THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+;;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+;;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+;;; ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+;;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+;;; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+;;; OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+;;; HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+;;; LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+;;; OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+;;; SUCH DAMAGE.
+;;;;
+
 (require "japanese.scm")
 (require "generic-key.scm")
 ;;
@@ -11,107 +41,31 @@
 (define anthy-segment-separator "|")
 
 ;; key defs
-(define anthy-on-key
-  (lambda (key key-state)
-    (or
-     (and (= (to-lower-char key) 106)
-	  (control-key-mask key-state))
-     (generic-on-key key key-state))))
-(define anthy-latin-key
-  (lambda (key key-state)
-    (or 
-     (and 
-      (= (to-lower-char key) 108)
-      (not (modifier-key-mask key-state)))
-     (generic-on-key key key-state))))
-(define anthy-wide-latin-key
-  (lambda (key key-state)
-    (and (= (to-lower-char key) 108)
-	 (shift-key-mask key-state))))
-(define anthy-hankaku-kana-key
-  (lambda (key key-state)
-    (and
-     (= (to-lower-char key) 113)
-     (control-key-mask key-state))))
-(define anthy-kana-toggle-key
-  (lambda (key key-state)
-    (and
-     (= (to-lower-char key) 113)
-     (not (modifier-key-mask key-state)))))
-(define anthy-commit-as-opposite-kana-key
-  (lambda (key key-state)
-    (and
-     (= (to-lower-char key) 113)
-     (shift-key-mask key-state))))
-(define anthy-begin-conv-key
-  (lambda (key key-state)
-    (= (to-lower-char key) 32)))
-(define anthy-commit-key
-  (lambda (key key-state)
-    (generic-commit-key key key-state)))
-(define anthy-extend-segment-key
-  (lambda (key key-state)
-    (or
-     (and
-      (control-key-mask key-state)
-      (= (to-lower-char key) 111))
-     (and
-      (shift-key-mask key-state)
-      (= key 'right)))))
-(define anthy-shrink-segment-key
-  (lambda (key key-state)
-    (or
-     (and
-      (control-key-mask key-state)
-      (= (to-lower-char key) 105))
-     (and
-      (shift-key-mask key-state)
-      (= key 'left)))))
-(define anthy-next-candidate-key
-  (lambda (key key-state)
-    (generic-next-candidate-key key key-state)))
-(define anthy-prev-candidate-key
-  (lambda (key key-state)
-    (generic-prev-candidate-key key key-state)))
-(define anthy-next-page-key
-  (lambda (key key-state)
-    (generic-next-page-key key key-state)))
-(define anthy-prev-page-key
-  (lambda (key key-state)
-    (generic-prev-page-key key key-state)))
-(define anthy-cancel-key
-  (lambda (key key-state)
-    (generic-cancel-key key key-state)))
-(define anthy-backspace-key
-  (lambda (key key-state)
-    (generic-backspace-key key key-state)))
-(define anthy-delete-key
-  (lambda (key key-state)
-    (generic-delete-key key key-state)))
-(define anthy-kill-key
-  (lambda (key key-state)
-    (generic-kill-key key key-state)))
-(define anthy-kill-backward-key
-  (lambda (key key-state)
-    (generic-kill-backward-key key key-state)))
-(define anthy-go-left-key
-  (lambda (key key-state)
-    (generic-go-left-key key key-state)))
-(define anthy-go-right-key
-  (lambda (key key-state)
-    (generic-go-right-key key key-state)))
-(define anthy-beginning-of-preedit-key
-  (lambda (key key-state)
-    (generic-beginning-of-preedit-key key key-state)))
-(define anthy-end-of-preedit-key
-  (lambda (key key-state)
-    (generic-end-of-preedit-key key key-state)))
-(define anthy-next-segment-key
-  (lambda (key key-state)
-    (generic-go-right-key key key-state)))
-(define anthy-prev-segment-key
-  (lambda (key key-state)
-    (generic-go-left-key key key-state)))
+(define-key anthy-on-key? '("<Control>j" "<Control>J" generic-on-key?))
+(define-key anthy-latin-key? '("l" generic-on-key?))
+(define-key anthy-wide-latin-key? "L")
+(define-key anthy-hankaku-kana-key? '("<Control>q" "<Control>Q"))
+(define-key anthy-kana-toggle-key? "q")
+(define-key anthy-commit-as-opposite-kana-key? "Q")
+(define-key anthy-begin-conv-key? " ")
+(define-key anthy-commit-key? 'generic-commit-key?)
+(define-key anthy-extend-segment-key? '("<Control>o" "<Control>O" "<Shift>right"))
+(define-key anthy-shrink-segment-key? '("<Control>i" "<Control>I" "<Shift>left"))
+(define-key anthy-next-candidate-key? 'generic-next-candidate-key?)
+(define-key anthy-prev-candidate-key? 'generic-prev-candidate-key?)
+(define-key anthy-next-page-key? 'generic-next-page-key?)
+(define-key anthy-prev-page-key? 'generic-prev-page-key?)
+(define-key anthy-cancel-key? 'generic-cancel-key?)
+(define-key anthy-backspace-key? 'generic-backspace-key?)
+(define-key anthy-delete-key? 'generic-delete-key?)
+(define-key anthy-kill-key? 'generic-kill-key?)
+(define-key anthy-kill-backward-key? 'generic-kill-backward-key?)
+(define-key anthy-go-left-key? 'generic-go-left-key?)
+(define-key anthy-go-right-key? 'generic-go-right-key?)
+(define-key anthy-beginning-of-preedit-key? 'generic-beginning-of-preedit-key?)
+(define-key anthy-end-of-preedit-key? 'generic-end-of-preedit-key?)
+(define-key anthy-next-segment-key? 'generic-go-right-key?)
+(define-key anthy-prev-segment-key? 'generic-go-left-key?)
 
 ;; access
 (define anthy-context-on
@@ -235,6 +189,10 @@
    (let ((c '())
 	 (rkc (rk-context-new ja-rk-rule #t #f)))
      (set! c (copy-list '(() () () () () () () () () () () 0 #t #f)))
+     (if (and
+	  (not anthy-init-lib-ok?)
+	  (symbol-bound? 'anthy-lib-init))
+	 (set! anthy-init-lib-ok? (anthy-lib-init)))
      (anthy-context-set-ac-id!
       c
       (if anthy-init-lib-ok?
@@ -250,18 +208,21 @@
     (im-commit-raw id)
     (anthy-context-set-commit-raw! ac #t)))
 
+(define anthy-opposite-kana
+  (lambda (kana)
+    (cond
+     ((= kana anthy-type-hiragana)
+      anthy-type-katakana)
+     ((= kana anthy-type-katakana)
+      anthy-type-hiragana)
+     ((= kana anthy-type-hankana)
+      anthy-type-hiragana))))
+
 (define anthy-context-kana-toggle
   (lambda (ac)
-    (let ((kana (anthy-context-kana-mode ac)))
-      (if (= kana anthy-type-hiragana)
-	  (anthy-context-set-kana-mode!
-	   ac anthy-type-katakana))
-      (if (= kana anthy-type-katakana)
-	  (anthy-context-set-kana-mode!
-	   ac anthy-type-hiragana))
-      (if (= kana anthy-type-hankana)
-	  (anthy-context-set-kana-mode!
-	   ac anthy-type-hiragana)))))
+    (let* ((kana (anthy-context-kana-mode ac))
+	   (opposite-kana (anthy-opposite-kana kana)))
+      (anthy-context-set-kana-mode! ac opposite-kana))))
 
 (define anthy-make-string
   (lambda (sl dir type)
@@ -375,7 +336,7 @@
   (lambda (c key key-state)
     (let ((id (context-id c))
 	  (ac (context-data c)))
-      (if (anthy-on-key key key-state)
+      (if (anthy-on-key? key key-state)
 	  (begin
 	    (anthy-begin-input ac)
 	    (anthy-update-mode ac id)
@@ -418,21 +379,21 @@
 	   (kana (anthy-context-kana-mode ac))
 	   (last "")
 	   (res))
-      ;; get resudiual 'n'
-       (set! res (rk-push-key-last! rkc))
+      ;; get residual 'n'
+      (set! res (rk-push-key-last! rkc))
       (if res
 	  (anthy-append-string ac res))
       ;;
       (anthy-lib-set-string
        ac-id
        (string-append
-		(anthy-make-left-string (anthy-context-left-string ac)  anthy-type-hiragana)
-		(anthy-make-right-string (anthy-context-right-string ac) anthy-type-hiragana)))
+	(anthy-make-left-string (anthy-context-left-string ac)  anthy-type-hiragana)
+	(anthy-make-right-string (anthy-context-right-string ac) anthy-type-hiragana)))
       (anthy-context-set-index-list!
        ac
        (anthy-make-index-list
-		(anthy-lib-get-nr-segments ac-id)
-		nil))
+	(anthy-lib-get-nr-segments ac-id)
+	nil))
       (anthy-context-set-state! ac #t)
       (anthy-context-set-cur-seg! ac 0)
       (rk-flush (anthy-context-rkc ac)))))
@@ -444,7 +405,7 @@
 	 (direct (ja-direct (charcode->string key))))
       (and
 
-       (if (anthy-wide-latin-key key key-state)
+       (if (anthy-wide-latin-key? key key-state)
 	   (begin
 	     (anthy-flush ac)
 	     (anthy-context-set-on! ac #f)
@@ -453,7 +414,7 @@
 	     (anthy-update-prop-label ac id)
 	     #f)
 	   #t)
-       (if (anthy-latin-key key key-state)
+       (if (anthy-latin-key? key key-state)
 	   (begin
 	     (anthy-flush ac)
 	     (anthy-context-set-on! ac #f)
@@ -462,24 +423,24 @@
 	     (anthy-update-prop-label ac id)
 	     #f)
 	   #t)
-       (if (anthy-backspace-key key key-state)
+       (if (anthy-backspace-key? key key-state)
 	   (begin
 	     (anthy-commit-raw ac id)
 	     #f)
 	   #t)
-       (if (anthy-delete-key key key-state)
+       (if (anthy-delete-key? key key-state)
 	   (begin
 	     (anthy-commit-raw ac id)
 	     #f)
 	   #t)
-       (if (anthy-hankaku-kana-key key key-state)
+       (if (anthy-hankaku-kana-key? key key-state)
 	   (begin 
 	     (anthy-context-set-kana-mode! ac anthy-type-hankana)
 	     (anthy-update-mode ac id)
 	     (anthy-update-prop-label ac id)
 	     #f)
 	   #t)
-       (if (anthy-kana-toggle-key key key-state)
+       (if (anthy-kana-toggle-key? key key-state)
 	   (begin 
 	     (anthy-context-kana-toggle ac)
 	     (anthy-update-mode ac id)
@@ -522,14 +483,14 @@
       (and
        ;; begin conversion
        (if (and
-	    (anthy-begin-conv-key key key-state)
-	    anthy-init-lib-ok?)
+	    (anthy-begin-conv-key? key key-state)
+	    (number? (anthy-context-ac-id ac)))
 	   (begin
 	     (anthy-begin-conv ac)
 	     #f)
 	   #t)
        ;; backspace
-       (if (anthy-backspace-key key key-state)
+       (if (anthy-backspace-key? key key-state)
 	   (begin
 	     (if (not (rk-backspace rkc))
 		 (if (anthy-context-left-string ac)
@@ -539,7 +500,7 @@
 	     #f)
 	   #t)
        ;; delete
-       (if (anthy-delete-key key key-state)
+       (if (anthy-delete-key? key key-state)
 	   (begin
 	     (if (not (rk-delete rkc))
 		 (if (anthy-context-right-string ac)
@@ -549,33 +510,33 @@
 	     #f)
 	   #t)
        ;; kill
-       (if (anthy-kill-key key key-state)
+       (if (anthy-kill-key? key key-state)
 	   (begin
 	     (anthy-context-set-right-string! ac ())
 	     #f)
 	   #t)
        ;; kill-backward
-       (if (anthy-kill-backward-key key key-state)
+       (if (anthy-kill-backward-key? key key-state)
 	   (begin
 	     (rk-flush rkc)
 	     (anthy-context-set-left-string! ac ())
 	     #f)
 	   #t)
        ;; Ҥ餬ʥ⡼ɤǥʤꤹ
-       (if (anthy-commit-as-opposite-kana-key key key-state)
+       (if (anthy-commit-as-opposite-kana-key? key key-state)
 	   (begin
 	     (im-commit
 	      id
 	      (string-append
 	       (anthy-make-left-string (anthy-context-left-string ac)
-				       (not kana))
+				       (anthy-opposite-kana kana))
 	       (anthy-make-right-string (anthy-context-right-string ac)
-					(not kana))))
+					(anthy-opposite-kana kana))))
 	     (anthy-flush ac)
 	     #f)
 	   #t)
        ;; ߤΤʤ塢Ҥ餬/ʥ⡼ɤڤ괹
-       (if (anthy-kana-toggle-key key key-state)
+       (if (anthy-kana-toggle-key? key key-state)
 	   (begin
 	     (im-commit
 	      id
@@ -591,49 +552,47 @@
 	     #f)
 	   #t)
        ;; cancel
-       (if (anthy-cancel-key key key-state)
+       (if (anthy-cancel-key? key key-state)
 	   (begin
 	     (anthy-flush ac)
 	     #f)
 	   #t)
        ;; commit
-       (if (anthy-commit-key key key-state)
+       (if (anthy-commit-key? key key-state)
 	   (begin
 	     (im-commit
 	      id
 	      (string-append
-		(anthy-make-left-string (anthy-context-left-string ac) kana)
-		(rk-pending rkc)
-		(anthy-make-right-string (anthy-context-right-string ac) kana)))
+	       (anthy-make-left-string (anthy-context-left-string ac) kana)
+	       (rk-pending rkc)
+	       (anthy-make-right-string (anthy-context-right-string ac) kana)))
 	     (anthy-flush ac)
 	     #f)
 	   #t)
        ;; left
-       (if (anthy-go-left-key key key-state)
+       (if (anthy-go-left-key? key key-state)
 	   (begin
 	     (if (anthy-context-left-string ac)
-		 (let
-		     ((c (car (anthy-context-left-string ac))))
+		 (let ((c (car (anthy-context-left-string ac))))
 		   (anthy-context-set-left-string!
 		    ac (cdr (anthy-context-left-string ac)))
-		   (anthy-context-set-right-string! ac
-						    (cons c
-							  (anthy-context-right-string ac)))))
+		   (anthy-context-set-right-string! 
+		    ac
+		    (cons c (anthy-context-right-string ac)))))
 	     #f)
 	   #t)
        ;; right
-       (if (anthy-go-right-key key key-state)
+       (if (anthy-go-right-key? key key-state)
 	   (begin
 	     (if (anthy-context-right-string ac)
-		 (let
-		     ((c (car (anthy-context-right-string ac))))
+		 (let ((c (car (anthy-context-right-string ac))))
 		   (anthy-context-set-right-string!
 		    ac (cdr (anthy-context-right-string ac)))
 		   (anthy-append-string ac c)))
 	     #f)
 	   #t)
        ;; beginning-of-preedit
-       (if (anthy-beginning-of-preedit-key key key-state)
+       (if (anthy-beginning-of-preedit-key? key key-state)
 	   (begin
 	     (if (anthy-context-left-string ac)
 		 (begin
@@ -646,7 +605,7 @@
 	     #f)
 	   #t)
        ;; end-of-preedit
-       (if (anthy-end-of-preedit-key key key-state)
+       (if (anthy-end-of-preedit-key? key key-state)
 	   (begin
 	     (if (anthy-context-right-string ac)
 		 (begin
@@ -656,6 +615,7 @@
 		     (reverse (anthy-context-right-string ac))
 		     (anthy-context-left-string ac)))
 		   (anthy-context-set-right-string! ac ())))
+;		   (rk-flush rkc)))
 	     #f)
 	   #t)
        ;; modifiers (except shift) => ignore
@@ -665,6 +625,11 @@
 	     (anthy-commit-raw ac id)
 	     #f)
 	   #t)
+  ;    (if (symbol? key)
+;	   (begin
+;	     (anthy-commit-raw ac id)
+;	     #f)
+;	   #t)
 ;       (set! key (to-lower-char key))
        (let* ((key-str (charcode->string key))
 	      (pend (rk-pending rkc))
@@ -768,19 +733,19 @@
 (define anthy-move-segment
   (lambda (ac dir)
     (let ((pos (+ (anthy-context-cur-seg ac) dir))
-	  (nseg (length  (anthy-context-index-list ac))))
+	  (nseg (length (anthy-context-index-list ac))))
       (if (and
 	   (> pos -1)
 	   (< pos nseg))
 	  (anthy-context-set-cur-seg! ac pos)))))
 
 (define anthy-move-candidate
-  (lambda (ac id off)
+  (lambda (ac id offset)
     (let* ((seg (anthy-context-cur-seg ac))
 	   (n (nth seg (anthy-context-index-list ac)))
 	   (ac-id (anthy-context-ac-id ac))
 	   (max (anthy-lib-get-nr-candidates ac-id seg)))
-      (set! n (+ n off))
+      (set! n (+ n offset))
       (if (>= n max)
 	  (set! n 0))
       (if (< n 0)
@@ -795,17 +760,17 @@
 	   anthy-use-candidate-window?)
 	  (begin
 	    (anthy-context-set-candidate-window! ac #t)
-	    (im-begin-candidate
+	    (im-activate-candidate-selector
 	     id
 	     max anthy-nr-candidate-max)))
       (if (anthy-context-candidate-window ac)
-	  (im-update-candidate id n)))))
+	  (im-select-candidate id n)))))
 
 (define anthy-reset-candidate-window
   (lambda (ac id)
     (if (anthy-context-candidate-window ac)
 	(begin
-	  (im-end-candidate id ac)
+	  (im-deactivate-candidate-selector id ac)
 	  (anthy-context-set-candidate-window! ac #f)))
     (anthy-context-set-candidate-op-count! ac 0)))
 
@@ -828,61 +793,73 @@
 (define anthy-proc-compose-state
   (lambda (ac id key key-state)
     (and
-     (if (anthy-commit-key key key-state)
+     (if (anthy-commit-key? key key-state)
 	 (begin
 	   (anthy-do-commit ac id)
 	   #f)
 	 #t)
-     (if (anthy-extend-segment-key key key-state)
+     (if (anthy-extend-segment-key? key key-state)
 	 (begin
 	   (anthy-resize-segment ac id 1)
 	   #f)
 	 #t)
-     (if (anthy-shrink-segment-key key key-state)
+     (if (anthy-shrink-segment-key? key key-state)
 	 (begin
 	   (anthy-resize-segment ac id -1)
 	   #f)
 	 #t)
-     (if (anthy-next-segment-key key key-state)
+     (if (anthy-next-segment-key? key key-state)
 	 (begin
 	   (anthy-move-segment ac 1)
 	   (anthy-reset-candidate-window ac id)
 	   #f)
 	 #t)
-     (if (anthy-prev-segment-key key key-state)
+     (if (anthy-prev-segment-key? key key-state)
 	 (begin
 	   (anthy-move-segment ac -1)
 	   (anthy-reset-candidate-window ac id)
 	   #f)
 	 #t)
-     (if (anthy-backspace-key key key-state)
+     (if (anthy-beginning-of-preedit-key? key key-state)
+	 (begin
+	   (anthy-context-set-cur-seg! ac 0)
+	   (anthy-reset-candidate-window ac id)
+	   #f)
+	 #t)
+     (if (anthy-end-of-preedit-key? key key-state)
+	 (let ((nseg (length (anthy-context-index-list ac))))
+	   (anthy-context-set-cur-seg! ac (- nseg 1))
+	   (anthy-reset-candidate-window ac id)
+	   #f)
+	 #t)
+     (if (anthy-backspace-key? key key-state)
 	 (begin
 	   (anthy-context-set-state! ac #f)
 	   (anthy-reset-candidate-window ac id)
 	   #f)
 	 #t)
-     (if (anthy-next-candidate-key key key-state)
+     (if (anthy-next-candidate-key? key key-state)
 	 (begin
 	   (anthy-move-candidate ac id 1)
 	   #f)
 	 #t)
-     (if (anthy-prev-candidate-key key key-state)
+     (if (anthy-prev-candidate-key? key key-state)
 	 (begin
 	   (anthy-move-candidate ac id -1)
 	   #f)
 	 #t)
-     (if (anthy-cancel-key key key-state)
+     (if (anthy-cancel-key? key key-state)
 	 (begin
 	   (anthy-context-set-state! ac #f)
 	   (anthy-reset-candidate-window ac id)
 	   #f)
 	 #t)
-     (if (anthy-prev-page-key key key-state)
+     (if (anthy-prev-page-key? key key-state)
 	 (begin
 	   (im-shift-page-candidate id #f)
 	   #f)
 	 #t)
-     (if (anthy-next-page-key key key-state)
+     (if (anthy-next-page-key? key key-state)
 	 (begin
 	   (im-shift-page-candidate id #t)
 	   #f)
@@ -899,26 +876,23 @@
 
 (define anthy-proc-wide-latin
   (lambda (c key key-state)
-    (let* ((w (ja-wide (charcode->string key)))
+    (let* ((char (charcode->string key))
+	   (w (or (ja-direct char)
+		  (ja-wide char)))
 	   (id (context-id c))
 	   (ac (context-data c)))
-      (and
-       (if (anthy-on-key key key-state)
-	   (begin
-	     (anthy-flush ac)
-	     (anthy-update-mode ac id)
-	     (anthy-update-prop-label ac id)
-	     #f)
-	   #t)
-       (if (and (modifier-key-mask key-state)
-		(not (shift-key-mask key-state)))
-	   (begin
-	     (anthy-commit-raw ac id)
-	     #f)
-	   #t)
-       (if w
-	   (im-commit id w)
-	   (anthy-commit-raw ac id)))
+      (cond
+       ((anthy-on-key? key key-state)
+	(anthy-flush ac)
+	(anthy-update-mode ac id)
+	(anthy-update-prop-label ac id))
+       ((and (modifier-key-mask key-state)
+	     (not (shift-key-mask key-state)))
+	(anthy-commit-raw ac id))
+       (w
+	(im-commit id w))
+       (else
+	(anthy-commit-raw ac id)))
       ())))
 
 (define anthy-press-key-handler
@@ -954,11 +928,6 @@
   (lambda (id)
     ()))
 
-(if (and
-     (symbol-bound? 'anthy-lib-init)
-     (anthy-lib-init))
-    (set! anthy-init-lib-ok? #t))
-
 (define anthy-get-candidate-handler
   (lambda (id idx accel-enum-hint)
     (let* ((c (find-context id))
@@ -976,7 +945,7 @@
 	   (ac-id (anthy-context-ac-id ac)))
       (set-car! (nthcdr seg (anthy-context-index-list ac)) idx)
 ;      (anthy-reset-candidate-window ac id)
-      (anthy-move-segment ac 1)
+;      (anthy-move-segment ac 1)
       (anthy-update-preedit ac id))))
 
 (define anthy-prop-handler
