@@ -1,31 +1,6 @@
 ;;; canna.scm: Canna for uim.
 ;;;
-;;; Copyright (c) 2003,2004 Anthy Project http://anthy.sourceforge.jp/
 ;;; Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
-;;; Copyright (c) 2003,2004 Masahito Omote <omote@utyuuzin.net>
-;;;
-;;; You can use, modify, distribute under the term of GPL or BSD Licence.
-;;;;
-;;;
-;;; This program is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 2 of the License, or
-;;; (at your option) any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU Library General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program; if not, write to the Free Software
-;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-;;;
-;;;;
-;;;
-;;; Copyright (c) 2003,2004 Anthy Project http://anthy.sourceforge.jp/
-;;; Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
-;;; Copyright (c) 2003,2004 Masahito Omote <omote@utyuuzin.net>
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -52,7 +27,6 @@
 ;;; LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 ;;; OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 ;;; SUCH DAMAGE.
-;;;
 ;;;;
 
 (require "japanese.scm")
@@ -71,74 +45,33 @@
 ;(define canna-server-name "localhost")
 ;(define canna-server-name "127.0.0.1")
 ;;; Key definition
-(define (canna-on-key? key key-state)
-  (and
-   (= key 92)
-   (control-key-mask key-state)))
-(define (canna-off-key? key key-state)
-  (and
-   (= key 92)
-   (control-key-mask key-state)))
-(define (canna-latin-key key key-state)
-  (and
-   (= key 92)
-   (control-key-mask key-state)))
-;(define (canna-latin-key key key-state)
-;  (or
-;   (and
-;    (= (to-lower-char key) 108)
-;    (not (modifier-key-mask key-state)))
-;   (generic-on-key key key-state)))
-(define (canna-wide-latin-key key key-state)
-  (and (= (to-lower-char key) 108)
-       (shift-key-mask key-state)))
-(define (canna-kana-toggle-key key key-state)
-  (and
-   (= (to-lower-char key) 113)
-   (not (modifier-key-mask key-state))))
-(define (canna-cancel-key key key-state)
-    (generic-cancel-key key key-state))
-(define (canna-backspace-key key key-state)
-  (generic-backspace-key key key-state))
-(define (canna-delete-key key key-state)
-  (generic-delete-key key key-state))
-(define (canna-go-left-key key key-state)
-  (generic-go-left-key key key-state))
-(define (canna-go-right-key key key-state)
-  (generic-go-right-key key key-state))
-(define (canna-commit-key key key-state)
-  (generic-commit-key key key-state))
-(define (canna-begin-conv-key? key key-state)
-  (or (= (to-lower-char key) 32)
-      (generic-on-key key key-state)))
-(define (canna-commit-key? key key-state)
-  (generic-commit-key key key-state))
-(define (canna-extend-segment-key? key key-state)
-  (or (and (control-key-mask key-state)
-	   (= (to-lower-char key) 111))
-      (and (shift-key-mask key-state)
-	   (= key 'right))))
-(define (canna-shrink-segment-key? key key-state)
-  (or (and (control-key-mask key-state)
-	   (= (to-lower-char key) 105))
-      (and (shift-key-mask key-state)
-	   (= key 'left))))
-(define (canna-next-candidate-key? key key-state)
-  (generic-next-candidate-key key key-state))
-(define (canna-prev-candidate-key? key key-state)
-  (generic-prev-candidate-key key key-state))
-(define (canna-cancel-key? key key-state)
-  (generic-cancel-key key key-state))
-(define (canna-delete-key? key key-state)
-  (generic-delete-key key key-state))
-(define (canna-go-left-key? key key-state)
-    (generic-go-left-key key key-state))
-(define (canna-go-right-key? key key-state)
-    (generic-go-right-key key key-state))
-(define (canna-next-segment-key? key key-state)
-    (generic-go-right-key key key-state))
-(define (canna-prev-segment-key? key key-state)
-    (generic-go-left-key key key-state))
+(define-key canna-on-key? "<Control>\\")
+(define-key canna-off-key? "<Control>\\")
+(define-key canna-latin-key "<Control>\\")
+;(define-key canna-latin-key '("l" generic-on-key?))
+(define-key canna-wide-latin-key? "L")
+;(define-key canna-hankaku-kana-key? '("<Control>q" "<Control>Q"))
+(define-key canna-kana-toggle-key? "q")
+;(define-key canna-commit-as-opposite-kana-key? "Q")
+(define-key canna-begin-conv-key? '(" " generic-on-key?))
+(define-key canna-commit-key? 'generic-commit-key?)
+(define-key canna-extend-segment-key? '("<Control>o" "<Control>O" "<Shift>right"))
+(define-key canna-shrink-segment-key? '("<Control>i" "<Control>I" "<Shift>left"))
+(define-key canna-next-candidate-key? 'generic-next-candidate-key?)
+(define-key canna-prev-candidate-key? 'generic-prev-candidate-key?)
+;(define-key canna-next-page-key? 'generic-next-page-key?)
+;(define-key canna-prev-page-key? 'generic-prev-page-key?)
+(define-key canna-cancel-key? 'generic-cancel-key?)
+(define-key canna-backspace-key? 'generic-backspace-key?)
+(define-key canna-delete-key? 'generic-delete-key?)
+;(define-key canna-kill-key? 'generic-kill-key?)
+;(define-key canna-kill-backward-key? 'generic-kill-backward-key?)
+(define-key canna-go-left-key? 'generic-go-left-key?)
+(define-key canna-go-right-key? 'generic-go-right-key?)
+;(define-key canna-beginning-of-preedit-key? 'generic-beginning-of-preedit-key?)
+;(define-key canna-end-of-preedit-key? 'generic-end-of-preedit-key?)
+(define-key canna-next-segment-key? 'generic-go-right-key?)
+(define-key canna-prev-segment-key? 'generic-go-left-key?)
 
 ;;; access
 (define (canna-context-id cc)
@@ -346,7 +279,7 @@
       ((rkc (canna-context-rkc cc))
        (direct (ja-direct (charcode->string key))))
     (and
-     (if (canna-wide-latin-key key key-state)
+     (if (canna-wide-latin-key? key key-state)
 	 (begin
 	   (canna-flush cc)
 	   (canna-context-set-on! cc #f)
@@ -355,7 +288,7 @@
 	   (canna-update-prop-label cc id)
 	   #f)
 	 #t)
-     (if (canna-latin-key key key-state)
+     (if (canna-latin-key? key key-state)
 	 (begin
 	   (canna-flush cc)
 	   (canna-context-set-on! cc #f)
@@ -364,17 +297,17 @@
 	   (canna-update-prop-label cc id)
 	   #f)
 	 #t)
-     (if (canna-backspace-key key key-state)
+     (if (canna-backspace-key? key key-state)
 	 (begin
 	   (im-commit-raw id)
 	   #f)
 	 #t)
-     (if (canna-delete-key key key-state)
+     (if (canna-delete-key? key key-state)
 	 (begin
 	   (im-commit-raw id)
 	   #f)
 	 #t)
-     (if (canna-kana-toggle-key key key-state)
+     (if (canna-kana-toggle-key? key key-state)
 	 (begin
 	   (canna-context-kana-toggle cc)
 	   (canna-update-mode cc id)
@@ -424,7 +357,7 @@
 	   #f)
 	 #t)
      ;; backspace
-     (if (canna-backspace-key key key-state)
+     (if (canna-backspace-key? key key-state)
 	 (begin
 	   (if (not (rk-backspace rkc))
 	       (if (canna-context-left-string cc)
@@ -434,7 +367,7 @@
 	   #f)
 	 #t)
      ;; delete
-     (if (canna-delete-key key key-state)
+     (if (canna-delete-key? key key-state)
 	 (begin
 	   (if (not (rk-delete rkc))
 	       (if (canna-context-right-string cc)
@@ -444,7 +377,7 @@
 	   #f)
 	 #t)
        ;; Ҥ餬ʥ⡼ɤǥʤꤹ
-     (if (canna-kana-toggle-key key key-state)
+     (if (canna-kana-toggle-key? key key-state)
 	 (begin
 	   (im-commit
 	    id
@@ -658,16 +591,16 @@
 	 canna-use-candidate-window?)
 	(begin
 	  (canna-context-set-candidate-window! cc #t)
-	  (im-begin-candidate
+	  (im-activate-candidate-selector
 	   id
 	   max canna-nr-candidate-max)))
     (if (canna-context-candidate-window cc)
-	(im-update-candidate id n))))
+	(im-select-candidate id n))))
 
 (define (canna-reset-candidate-window cc id)
   (if (canna-context-candidate-window cc)
       (begin
-	(im-end-candidate id cc)
+	(im-deactivate-candidate-selector id cc)
 	(canna-context-set-candidate-window! cc #f)))
   (canna-context-set-candidate-op-count! cc 0))
 
@@ -690,7 +623,7 @@
   (let
       ((cc-id (canna-context-cc-id cc)))
     (and
-     (if (canna-commit-key key key-state)
+     (if (canna-commit-key? key key-state)
 	 (begin
 	   (canna-do-commit cc id)
 	   #f)
@@ -717,7 +650,7 @@
 	   (canna-reset-candidate-window cc id)
 	   #f)
 	 #t)
-     (if (canna-backspace-key key key-state)
+     (if (canna-backspace-key? key key-state)
 	 (begin
 	   (canna-context-set-state! cc #f)
 	   (canna-reset-candidate-window cc id)
@@ -750,26 +683,23 @@
     ()))
 
 (define (canna-proc-wide-latin c key key-state)
-  (let* ((w (ja-wide (charcode->string key)))
+  (let* ((char (charcode->string key))
+	 (w (or (ja-direct char)
+		(ja-wide char)))
 	 (id (context-id c))
 	 (cc (context-data c)))
-    (and
-     (if (canna-on-key? key key-state)
-	 (begin
-	   (canna-flush cc)
-	   (canna-update-mode cc id)
-	   (canna-update-prop-label cc id)
-	   #f)
-	 #t)
-     (if (and (modifier-key-mask key-state)
-	      (not (shift-key-mask key-state)))
-	 (begin
-	   (im-commit-raw id)
-	   #f)
-	 #t)
-     (if w
-	 (im-commit id w)
-	 (im-commit-raw id)))
+    (cond
+     ((canna-on-key? key key-state)
+      (canna-flush cc)
+      (canna-update-mode cc id)
+      (canna-update-prop-label cc id))
+     ((and (modifier-key-mask key-state)
+	   (not (shift-key-mask key-state)))
+      (im-commit-raw id))
+     (w
+      (im-commit id w))
+     (else
+      (im-commit-raw id)))
     ()))
 
 (define (canna-press-key-handler id key key-state)
