@@ -1,3 +1,33 @@
+;;;
+;;; Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+;;;
+;;; All rights reserved.
+;;;
+;;; Redistribution and use in source and binary forms, with or without
+;;; modification, are permitted provided that the following conditions
+;;; are met:
+;;; 1. Redistributions of source code must retain the above copyright
+;;;    notice, this list of conditions and the following disclaimer.
+;;; 2. Redistributions in binary form must reproduce the above copyright
+;;;    notice, this list of conditions and the following disclaimer in the
+;;;    documentation and/or other materials provided with the distribution.
+;;; 3. Neither the name of authors nor the names of its contributors
+;;;    may be used to endorse or promote products derived from this software
+;;;    without specific prior written permission.
+;;;
+;;; THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+;;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+;;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+;;; ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+;;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+;;; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+;;; OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+;;; HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+;;; LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+;;; OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+;;; SUCH DAMAGE.
+;;;;
+
 (require "rk.scm")
 (require "generic-key.scm")
 ;;
@@ -74,7 +104,7 @@
 	   (cs (cadr (rk-current-seq rkc)))
 	   (res))
       (and
-       (if (generic-off-key key state)
+       (if (generic-off-key? key state)
 	   (begin
 	     (rk-flush rkc)
 	     (generic-context-set-on! pc #f)
@@ -82,52 +112,64 @@
 	     (im-update-mode id 0)
 	     #f)
 	   #t)
-       (if (generic-prev-candidate-key key state)
-	   (begin
-	     (set! n (- n 1))
-	     (generic-context-set-rk-nth! pc n)
-	     (if (< n 0)
-		 (begin
-		   (generic-context-set-rk-nth! pc (- (length cs) 1))
-		   (set! n (- (length cs) 1))))
-	     (generic-context-set-candidate-op-count!
-	      pc
-	      (+ 1 (generic-context-candidate-op-count pc)))
-	     (if (>= (generic-context-candidate-op-count pc)
-		     generic-candidate-op-count)
-		 (im-update-candidate id n))
-	     #f)
+       (if (generic-prev-candidate-key? key state)
+	   (if (> (length (cadr cs)) 0)
+	       (begin
+		 (set! n (- n 1))
+		 (generic-context-set-rk-nth! pc n)
+		 (if (< n 0)
+		     (begin
+		       (generic-context-set-rk-nth! pc (- (length cs) 1))
+		       (set! n (- (length cs) 1))))
+		 (generic-context-set-candidate-op-count!
+		  pc
+		  (+ 1 (generic-context-candidate-op-count pc)))
+		 (if (>= (generic-context-candidate-op-count pc)
+			 generic-candidate-op-count)
+		     (im-select-candidate id n))
+		 #f)
+	       (begin
+		 (im-commit-raw id)
+		 (rk-flush rkc)
+		 (im-update-preedit id)
+		 #f))
 	   #t)
-       (if (generic-next-candidate-key key state)
-	   (begin
-	     (generic-context-set-rk-nth! pc (+ 1 n))
-	     (if (<= (length cs) (+ n 1))
-		 (generic-context-set-rk-nth! pc 0))
-	     (generic-context-set-candidate-op-count!
-	      pc
-	      (+ 1 (generic-context-candidate-op-count pc)))
-	     (if (and
-		  (= (generic-context-candidate-op-count pc)
-		     generic-candidate-op-count)
-		  generic-use-candidate-window?)
-		 (im-begin-candidate id (length cs) generic-nr-candidate-max))
-	     (if (>= (generic-context-candidate-op-count pc)
-		     generic-candidate-op-count)
-		 (begin
-		   (if (>= (+ n 1) (length cs))
-		       (set! n -1))
-		   (im-update-candidate id (+ n 1))))
-	     #f)
+       (if (generic-next-candidate-key? key state)
+	   (if (> (length (cadr cs)) 0)
+	       (begin
+		 (generic-context-set-rk-nth! pc (+ 1 n))
+		 (if (<= (length cs) (+ n 1))
+		     (generic-context-set-rk-nth! pc 0))
+		 (generic-context-set-candidate-op-count!
+		  pc
+		  (+ 1 (generic-context-candidate-op-count pc)))
+		 (if (and
+		      (= (generic-context-candidate-op-count pc)
+			 generic-candidate-op-count)
+		      generic-use-candidate-window?)
+		     (im-activate-candidate-selector id (length cs) generic-nr-candidate-max))
+		 (if (>= (generic-context-candidate-op-count pc)
+			 generic-candidate-op-count)
+		     (begin
+		       (if (>= (+ n 1) (length cs))
+			   (set! n -1))
+		       (im-select-candidate id (+ n 1))))
+		 #f)
+	       (begin
+		 (im-commit-raw id)
+		 (rk-flush rkc)
+		 (im-update-preedit id)
+		 #f))
 	   #t)
-       (if (generic-backspace-key key state)
+       (if (generic-backspace-key? key state)
 	   (begin
 	     (if (not (rk-backspace rkc))
 		 (generic-commit-raw pc id))
 	     (generic-context-set-rk-nth! pc 0)
-	     (im-end-candidate id)
+	     (im-deactivate-candidate-selector id)
 	     #f)
 	   #t)
-       (if (generic-commit-key key state)
+       (if (generic-commit-key? key state)
 	   (let ((cs (rk-current-seq rkc)))
 	     (if (> (length (cadr cs)) 0)
 		 (begin
@@ -135,9 +177,10 @@
 		   (generic-context-set-rk-nth! pc 0)
 		   (generic-context-set-candidate-op-count! pc 0)
 		   (rk-flush rkc)
-		   (im-end-candidate id))
+		   (im-deactivate-candidate-selector id))
 		 (begin
-		   (rk-flush! rkc)
+		   (im-commit-raw id)
+		   (rk-flush rkc)
 		   (im-update-preedit id)))
 	     #f)
 	   #t)
@@ -148,6 +191,12 @@
 	     (generic-context-set-rk-nth! pc 0)
 	     #f)
 	   #t)
+       (if (and (modifier-key-mask state)
+		(not (shift-key-mask state)))
+	   (begin
+	     (generic-commit-raw pc id)
+	     #f)
+	   #t)
        (begin
 	 (set! res
 	       (rk-push-key!
@@ -162,20 +211,20 @@
 			     (nth (generic-context-rk-nth pc) (cadr cs)))
 		  (generic-context-set-rk-nth! pc 0)
 		  (generic-context-set-candidate-op-count! pc 0)
-		  (im-end-candidate id)
+		  (im-deactivate-candidate-selector id)
 		  (rk-flush rkc)))))
       (if res
 	  (begin
 	    (im-commit id (nth (generic-context-rk-nth pc) res))
 	    (generic-context-set-rk-nth! pc 0)
 	    (generic-context-set-candidate-op-count! pc 0)
-	    (im-end-candidate id))
+	    (im-deactivate-candidate-selector id))
 	  ))))
 
 (define generic-proc-off-mode
   (lambda (id pc key state)
     (and
-     (if (generic-on-key key state)
+     (if (generic-on-key? key state)
 	 (begin
 	   (generic-context-set-on! pc #t)
 	   (generic-update-prop-label pc id)
