@@ -1,3 +1,33 @@
+;;;
+;;; Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+;;;
+;;; All rights reserved.
+;;;
+;;; Redistribution and use in source and binary forms, with or without
+;;; modification, are permitted provided that the following conditions
+;;; are met:
+;;; 1. Redistributions of source code must retain the above copyright
+;;;    notice, this list of conditions and the following disclaimer.
+;;; 2. Redistributions in binary form must reproduce the above copyright
+;;;    notice, this list of conditions and the following disclaimer in the
+;;;    documentation and/or other materials provided with the distribution.
+;;; 3. Neither the name of authors nor the names of its contributors
+;;;    may be used to endorse or promote products derived from this software
+;;;    without specific prior written permission.
+;;;
+;;; THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+;;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+;;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+;;; ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+;;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+;;; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+;;; OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+;;; HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+;;; LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+;;; OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+;;; SUCH DAMAGE.
+;;;;
+
 ;;TODO
 ;;    ::ñϿ
 ;;
@@ -27,62 +57,26 @@
 ;; config function
 (define prime-dont-use-numeral-key-to-select-cand
   (lambda ()
-    (set! prime-cand-select-key
+    (set! prime-cand-select-key?
 	  (lambda (key key-state)
 	    (and (numeral? key)
 		 (control-key-mask key-state))))))
 
 ;; key
-(define prime-latin-key
-  (lambda (key key-state)
-    (or
-     (generic-off-key key key-state)
-     (and (= (to-lower-char key) 108)
-	  (control-key-mask key-state)))))
-(define prime-wide-latin-key
-  (lambda (key key-state)
-    (and (= (to-lower-char key) 108)
-	 (shift-key-mask key-state))))
-(define prime-begin-conv-key
-  (lambda (key key-state)
-    (= key 32)))
-(define prime-on-key
-  (lambda (key key-state)
-    (or
-     (and (= (to-lower-char key) 106)
-	  (control-key-mask key-state))
-     (generic-on-key key key-state))))
-(define prime-commit-key
-  (lambda (key key-state)
-    (generic-commit-key key key-state)))
-(define prime-next-candidate
-  (lambda (key key-state)
-    (generic-next-candidate-key key key-state)))
-(define prime-prev-candidate
-  (lambda (key key-state)
-    (or
-     (generic-prev-candidate-key key key-state)
-     (= (to-lower-char key) 120))))
-(define prime-kana-toggle
-  (lambda (key key-state)
-    (and (= (to-lower-char key) 113)
-	 (control-key-mask key-state))))
-(define prime-cancel-key
-  (lambda (key key-state)
-    (generic-cancel-key key key-state)))
-(define prime-backspace-key
-  (lambda (key key-state)
-    (generic-backspace-key key key-state)))
-(define prime-delete-key
-  (lambda (key key-state)
-    (generic-delete-key key key-state)))
-(define prime-go-left-key
-  (lambda (key key-state)
-    (generic-go-left-key key key-state)))
-(define prime-go-right-key
-  (lambda (key key-state)
-    (generic-go-right-key key key-state)))
-(define prime-cand-select-key
+(define-key prime-latin-key? '("<Control>l" "<Control>L" generic-off-key?))
+(define-key prime-wide-latin-key? nil)
+(define-key prime-begin-conv-key? " ")
+(define-key prime-on-key? '("<Control>j" "<Control>J" generic-on-key?))
+(define-key prime-commit-key? 'generic-commit-key?)
+(define-key prime-next-candidate-key? 'generic-next-candidate-key?)
+(define-key prime-prev-candidate-key? 'generic-prev-candidate-key?)
+(define-key prime-kana-toggle-key? '("<Control>q" "<Control>Q"))
+(define-key prime-cancel-key? 'generic-cancel-key?)
+(define-key prime-backspace-key? 'generic-backspace-key?)
+(define-key prime-delete-key? 'generic-delete-key?)
+(define-key prime-go-left-key? 'generic-go-left-key?)
+(define-key prime-go-right-key? 'generic-go-right-key?)
+(define prime-cand-select-key?
   (lambda (key key-state)
     (numeral? key)))
 
@@ -185,6 +179,10 @@
     (set-car! (nthcdr 0 c) s)))
 ;; state kana head okuri tail candidates nth rk
 
+(define prime-mode-direct 0)
+(define prime-mode-hiragana 1)
+(define prime-mode-zenkaku 2)
+
 (define prime-flush
   (lambda (sc)
     (rk-flush (prime-context-rk-context sc))
@@ -208,14 +206,27 @@
       (prime-flush c)
       c)))
 
+;; which is responsible to determine input mode?  -- YamaKen
+;;   - prime-context-mode
+;;   - prime-context-kana-mode and prime-context-wide-latin
+;;   - prime-context-state
 (define prime-context-kana-toggle
   (lambda (sc)
-    (let ((s (prime-context-kana-mode sc)))
-      (set! s (not s))
-      (if s
-	  (prime-context-set-mode! sc 1)
-	  (prime-context-set-mode! sc 2))
-      (prime-context-set-kana-mode! sc s))))
+    (let ((kana (not (prime-context-kana-mode sc))))
+      (cond
+       (kana
+	(prime-context-set-state! sc 'prime-state-no-preedit)
+	(prime-context-set-mode! sc prime-mode-hiragana)
+	(prime-context-set-kana-mode! sc #t))
+       ((prime-context-wide-latin sc)
+	(prime-context-set-state! sc 'prime-state-direct)
+	(prime-context-set-mode! sc prime-mode-zenkaku)
+	(prime-context-set-kana-mode! sc #f))
+       ((not (prime-context-wide-latin sc))
+	(prime-context-set-state! sc 'prime-state-wide-latin)
+	(prime-context-set-mode! sc prime-mode-direct)
+	(prime-context-set-kana-mode! sc #f)))
+      (prime-context-set-kana-mode! sc kana))))
 
 (define prime-get-nth-candidate
   (lambda (sc n)
@@ -288,8 +299,8 @@
        (string-split str-line "\t"))
      (cdr (delq "" (string-split cands-string "\n"))))))
 
-(define prime-begin-conversion
-  (lambda (sc id)
+(define prime-begin-conversion-internal
+  (lambda (sc id init-idx)
     (let ((res)
 	  (rkc (prime-context-rk-context sc)))
       (prime-get-candidates!
@@ -299,15 +310,25 @@
        (rk-pending rkc))
       (prime-context-last-word sc)
       (set! res
-	    (prime-get-nth-candidate sc 0))
+	    (prime-get-nth-candidate sc init-idx))
       (if res
 	  (begin
-	    (prime-context-set-nth! sc 0)
+	    (prime-context-set-nth! sc init-idx)
 	    (prime-context-set-state!
 	     sc 'prime-state-converting))
 	  (prime-flush sc))
       )))
 
+(define prime-begin-conversion-reversely
+  (lambda (sc id)
+    (let ((last-idx (- (prime-get-nr-candidates sc)
+		       1)))
+      (prime-begin-conversion-internal sc id last-idx))))
+
+(define prime-begin-conversion
+  (lambda (sc id)
+    (prime-begin-conversion-internal sc id 0)))
+
 (define prime-update-preedit
   (lambda (id sc)
     (if (not (prime-context-raw-commit sc))
@@ -380,7 +401,7 @@
 				   (string-list-concat (prime-context-right-string sc)))))
       (cond
        ((> nth 0)
-	(im-update-candidate id nth))
+	(im-select-candidate id nth))
        ((and
 	 (prime-has-preedit? sc)
 	 (not (prime-context-candidate-window sc))
@@ -394,14 +415,14 @@
 	   sc
 	   preedit
 	   (prime-context-last-word sc))
-	  (im-begin-candidate
+	  (im-activate-candidate-selector
 	   id (prime-get-nr-candidates sc) prime-nr-candidate-max)
 	  (prime-context-set-candidate-window! sc #t)))
 	  
        ((not (prime-has-preedit? sc))
 	(begin
 	  (prime-context-set-candidate-window! sc #f)
-	  (im-end-candidate id)))
+	  (im-deactivate-candidate-selector id)))
 	  
        ((prime-context-candidate-window sc)
 	(begin
@@ -409,8 +430,9 @@
 	   sc
 	   preedit
 	   (prime-context-last-word sc))
-	  (im-update-candidate id nth)))
-    ))))
+	  (if (= (prime-context-state sc)
+		 'prime-state-converting)
+	      (im-select-candidate id nth))))))))
 
 (define prime-has-preedit?
   (lambda (sc)
@@ -454,38 +476,38 @@
 	   (registered (string-append (string-list-concat (prime-context-left-left-strings sc))
 				(string-list-concat (prime-context-right-right-strings sc)))))
       (cond
-       ((prime-wide-latin-key key key-state)
+       ((prime-wide-latin-key? key key-state)
 	(begin
-	  (prime-context-set-mode! sc 3)
+	  (prime-context-set-mode! sc prime-mode-zenkaku)
 	  (prime-context-set-state! sc 'prime-state-wide-latin)
 	  (prime-update-mode id sc)))
        
-       ((prime-latin-key key key-state)
+       ((prime-latin-key? key key-state)
 	(begin
-	  (prime-context-set-mode! sc 0)
+	  (prime-context-set-mode! sc prime-mode-direct)
 	  (prime-context-set-state! sc 'prime-state-latin)
 	  (prime-update-mode id sc)))
        
-       ((prime-kana-toggle key key-state)
+       ((prime-kana-toggle-key? key key-state)
 	(begin 
 	  (prime-context-kana-toggle sc)
 	  (prime-update-mode id sc)))
        
-       ((prime-backspace-key key key-state)
+       ((prime-backspace-key? key key-state)
 	(if (not (rk-backspace rkc))
 	    (if (prime-context-left-left-strings sc)
 		(prime-context-set-left-left-strings!
 		 sc (cdr (prime-context-left-left-strings sc)))
 		(prime-commit-raw sc id))))
 
-       ((prime-delete-key key key-state)
+       ((prime-delete-key? key key-state)
 	(if (not (rk-delete rkc))
 	    (if (prime-context-right-right-strings sc)
 		(prime-context-set-right-right-strings!
 		 sc (cdr (prime-context-right-right-strings sc)))
 		(prime-commit-raw sc id))))
 
-       ((prime-cancel-key key key-state)
+       ((prime-cancel-key? key key-state)
 	(if learning-word
 	    (begin
 	      (prime-context-set-left-string!
@@ -494,7 +516,7 @@
 	      (prime-context-set-left-left-strings! sc '())
 	      (prime-context-set-right-right-strings! sc '()))))
        
-       ((prime-commit-key key key-state)
+       ((prime-commit-key? key key-state)
 	(if (and learning-word
 		 (not (string-equal? registered "")))
 	    (begin
@@ -511,12 +533,12 @@
 	  (prime-context-set-immediate-commit! sc #f)
 	  (prime-proc-input-with-preedit c key key-state)))
 
-       ((prime-cand-select-key key key-state)
+       ((prime-cand-select-key? key key-state)
 	(begin
 	  (prime-context-set-immediate-commit! sc #f)
 	  (prime-proc-input-with-preedit c key key-state)))
 
-       ((prime-go-left-key key key-state)
+       ((prime-go-left-key? key key-state)
 	(begin
 	  (if (prime-context-left-left-strings sc)
 	      (let ((c (car (prime-context-left-left-strings sc))))
@@ -528,7 +550,7 @@
 	      (prime-commit-raw sc id))))
  
        ;; right
-       ((prime-go-right-key key key-state)
+       ((prime-go-right-key? key key-state)
 	(begin
 	  (if (prime-context-right-right-strings sc)
 	      (let ((c (car (reverse (prime-context-right-right-strings sc)))))
@@ -571,13 +593,17 @@
 	   (res ())
 	   (learning-word (prime-context-learning-word sc)))
       (cond
-       ((prime-begin-conv-key key key-state)
+       ((or (prime-next-candidate-key? key key-state)
+	    (prime-begin-conv-key? key key-state))
 	(prime-begin-conversion sc id))
-       
-       ((prime-cancel-key key key-state)
+
+       ((prime-prev-candidate-key? key key-state)
+	(prime-begin-conversion-reversely sc id))
+
+       ((prime-cancel-key? key key-state)
 	())
        
-       ((prime-backspace-key key key-state)
+       ((prime-backspace-key? key key-state)
 	(begin
 	  (if (prime-context-left-string sc)
 	      (prime-context-set-left-string!
@@ -585,7 +611,7 @@
 	  (prime-context-set-candidate-window! sc #f) ;FIXME:very dirty hack
 	  ))
        ;; delete
-       ((prime-delete-key key key-state)
+       ((prime-delete-key? key key-state)
 	(begin
 	  (if (prime-context-right-string sc)
 	      (prime-context-set-right-string!
@@ -595,7 +621,7 @@
 	  (prime-context-set-candidate-window! sc #f) ;FIXME:very dirty hack
 	  ))
        ;; commit
-       ((prime-commit-key key key-state)
+       ((prime-commit-key? key key-state)
 	(if learning-word
 	    (prime-commit-to-left-left sc
 				       (append
@@ -608,7 +634,7 @@
 	      (prime-update-mode id sc))))
        
        ;; left
-       ((prime-go-left-key key key-state)
+       ((prime-go-left-key? key key-state)
 	(begin
 	  (if (prime-context-left-string sc)
 	      (let ((c (car (prime-context-left-string sc))))
@@ -619,7 +645,7 @@
 		 (append (prime-context-right-string sc) (list c)))))))
        
        ;; right
-       ((prime-go-right-key key key-state)
+       ((prime-go-right-key? key key-state)
 	(begin
 	  (if (prime-context-right-string sc)
 	      (let ((c (car (reverse (prime-context-right-string sc)))))
@@ -629,7 +655,7 @@
 		 sc
 		 (append (list c) (prime-context-left-string sc)))))))
 		
-       ((and (prime-cand-select-key key key-state)
+       ((and (prime-cand-select-key? key key-state)
 	     (prime-context-immediate-commit sc))
 	(let* ((nth (- key 49))
 	       (cand (prime-get-nth-candidate sc nth)))
@@ -650,10 +676,6 @@
 	 (modifier-key-mask key-state)
 	 (not (shift-key-mask key-state)))
 	(prime-commit-raw sc id))
-       
-       ((or (prime-prev-candidate key key-state)
-	    (prime-next-candidate key key-state))
-	(prime-proc-state-converting c key key-state))
 
        (else
 	(begin
@@ -681,7 +703,7 @@
 	  (res ())
 	  (learning-word (prime-context-learning-word sc)))
       (cond
-       ((prime-next-candidate key key-state)
+       ((prime-next-candidate-key? key key-state)
 	(begin
 	  (prime-context-set-nth! sc
 				  (+ 1 (prime-context-nth sc)))
@@ -693,16 +715,16 @@
 		  (prime-context-set-nth! sc 0)
 		  (prime-word-learning-start! sc)))))
 
-       ((prime-prev-candidate key key-state)
+       ((prime-prev-candidate-key? key key-state)
 	(begin
 	  (if (> (prime-context-nth sc) 0)
 	      (prime-context-set-nth! sc (- (prime-context-nth sc) 1))
 	      (prime-context-set-nth! sc (- (prime-get-nr-candidates sc) 1)))))
 
-       ((prime-cancel-key key key-state)
+       ((prime-cancel-key? key key-state)
 	(prime-flush sc))
        
-       ((prime-commit-key key key-state)
+       ((prime-commit-key? key key-state)
 	(if learning-word
 	    (prime-commit-to-left-left sc (string-to-list (prime-get-nth-candidate sc (prime-context-nth sc))))
 	    (begin
@@ -711,7 +733,7 @@
 	      (prime-flush sc)
 	      (prime-update-mode id sc))))
 
-       ((prime-cand-select-key key key-state)
+       ((prime-cand-select-key? key key-state)
 	(let* ((nth (- key 49))
 	       (cand (prime-get-nth-candidate sc nth)))
 	  (if cand
@@ -748,21 +770,23 @@
   (lambda (c key key-state)
     (let ((sc (context-data c))
 	  (id (context-id c)))
-      (if (prime-on-key key key-state)
+      (if (prime-on-key? key key-state)
 	  (begin
-	    (prime-context-set-mode! sc 1)
+	    (prime-context-set-mode! sc prime-mode-hiragana)
 	    (prime-update-mode id sc))
 	  (prime-commit-raw sc id)))))
 
 (define prime-proc-mode-wide-latin
   (lambda (c key key-state)
-    (let* ((w (ja-wide (charcode->string key)))
+    (let* ((char (charcode->string key))
+	   (w (or (ja-direct char)
+		  (ja-wide char)))
 	   (id (context-id c))
 	   (sc (context-data c)))
-      (if (prime-on-key key key-state)
+      (if (prime-on-key? key key-state)
 	  (begin
 	    (prime-flush sc)
-	    (prime-context-set-mode! sc 1)
+	    (prime-context-set-mode! sc prime-mode-hiragana)
 	    (prime-update-mode id sc))
 	  (if w
 	      (im-commit id w)
@@ -775,17 +799,19 @@
 	   (mode (prime-context-mode sc))
 	   (fun)
 	   (res))
-      (if (= mode 0)
-	  (set! fun prime-proc-mode-latin))
-      (if (= mode 3)
-	  (set! fun prime-proc-mode-wide-latin))
-      (if (or (= mode 1) (= mode 2))
-	  (begin
-	    (if (prime-has-preedit? sc)
-		(set! fun prime-proc-input-with-preedit)
-		(set! fun prime-proc-input-no-preedit))
-	    (if (= state 'prime-state-converting)
-		(set! fun prime-proc-state-converting))))
+      (cond
+       ((= mode prime-mode-direct)
+	(set! fun prime-proc-mode-latin))
+       ((= mode prime-mode-zenkaku)
+	(set! fun prime-proc-mode-wide-latin))
+      ;((or (= mode 1) (= mode 2))  ;; what's intented? -- YamaKen
+      ((= mode prime-mode-hiragana)
+       (begin
+	 (if (prime-has-preedit? sc)
+	     (set! fun prime-proc-input-with-preedit)
+	     (set! fun prime-proc-input-no-preedit))
+	 (if (= state 'prime-state-converting)
+	     (set! fun prime-proc-state-converting)))))
       (fun c key key-state)
       (prime-update-candidate-window sc (context-id c))
       (prime-update-preedit (context-id c) sc)      
@@ -802,7 +828,7 @@
       (im-pushback-mode-list id "Ҥ餬")
       (im-pushback-mode-list id "ѱѿ")
       (im-update-mode-list id)
-      (im-update-mode id 0)
+      (im-update-mode id prime-mode-direct)
 
       (prime-update-prop-list id))))
 
@@ -831,10 +857,17 @@
 	   (sc (context-data c)))
       (prime-flush sc)
       (prime-context-set-mode! sc mode)
-      (if (= mode 1)
-	  (prime-context-set-kana-mode! sc #t))
-      (if (= mode 2)
-	  (prime-context-set-kana-mode! sc #f))
+      (cond
+       ((= mode prime-mode-direct)
+	(begin
+	  (prime-context-set-kana-mode! sc #f)
+	  (prime-context-set-wide-latin! sc #f)))
+       ((= mode prime-mode-hiragana)
+	(prime-context-set-kana-mode! sc #t))
+       ((= mode prime-mode-zenkaku)
+	(begin
+	  (prime-context-set-kana-mode! sc #f)
+	  (prime-context-set-wide-latin! sc #t))))
       (prime-update-preedit id sc)
       ())))
 
@@ -861,14 +894,19 @@
        ((string-equal? message
 		       "prop_prime_hiragana")
 	(begin
+	  (prime-context-set-mode! sc prime-mode-hiragana)
 	  (prime-context-set-kana-mode! sc #t)))
        ((string-equal? message
 		       "prop_prime_direct")
 	(begin
+	  (prime-context-set-mode! sc prime-mode-direct)
+	  (prime-context-set-kana-mode! sc #f)  ;; required? -- YamaKen
 	  (prime-context-set-wide-latin! sc #f)))
        ((string-equal? message
 		       "prop_prime_zenkaku")
 	(begin
+	  (prime-context-set-mode! sc prime-mode-zenkaku)
+	  (prime-context-set-kana-mode! sc #f)  ;; required? -- YamaKen
 	  (prime-context-set-wide-latin! sc #t))))
       (prime-update-mode id sc)
       (prime-update-prop-label id sc))))
