@@ -1,3 +1,33 @@
+;;;
+;;; Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+;;;
+;;; All rights reserved.
+;;;
+;;; Redistribution and use in source and binary forms, with or without
+;;; modification, are permitted provided that the following conditions
+;;; are met:
+;;; 1. Redistributions of source code must retain the above copyright
+;;;    notice, this list of conditions and the following disclaimer.
+;;; 2. Redistributions in binary form must reproduce the above copyright
+;;;    notice, this list of conditions and the following disclaimer in the
+;;;    documentation and/or other materials provided with the distribution.
+;;; 3. Neither the name of authors nor the names of its contributors
+;;;    may be used to endorse or promote products derived from this software
+;;;    without specific prior written permission.
+;;;
+;;; THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+;;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+;;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+;;; ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+;;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+;;; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+;;; OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+;;; HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+;;; LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+;;; OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+;;; SUCH DAMAGE.
+;;;;
+
 ;; SKKκƵؽ򥤥饤ǹԤδʰץƥȥǥ
 ;;
 (define skk-editor-new
@@ -92,28 +122,30 @@
 	  (skk-editor-commit ec raw-str)
 	  ;; not a string
 	  (and
-	   (if (skk-backspace-key key key-state)
+	   (if (skk-backspace-key? key key-state)
 	       (let ((cur (skk-editor-left-string ec)))
 		 (if cur
 		     (skk-editor-set-left-string!
 		      ec (cdr cur)))
 		 #f)
 	       #t)
-	   (if (skk-return-key key key-state)
+	   (if (skk-return-key? key key-state)
 	       (begin
 		 (if (< 0 (string-length str))
 		     (begin
 		       (skk-lib-learn-word
-			(skk-make-string (skk-context-head sc) #t)
+			(skk-make-string (skk-context-head sc)
+					 skk-type-hiragana)
 			(skk-context-okuri-head sc)
-			(skk-make-string (skk-context-okuri sc) #t)
+			(skk-make-string (skk-context-okuri sc)
+					 skk-type-hiragana)
 			str)
 		       (skk-save-personal-dictionary)))
 		 (skk-commit-editor-context
 		  sc (skk-editor-id ec) str)
 		 #f)
 	       #t)
-	   (if (skk-cancel-key key key-state)
+	   (if (skk-cancel-key? key key-state)
 	       (begin
 		 (skk-back-to-kanji-state sc (skk-editor-id ec))
 		 (skk-context-set-child-context! sc nil)
