@@ -1,3 +1,33 @@
+;;;
+;;; Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+;;;
+;;; All rights reserved.
+;;;
+;;; Redistribution and use in source and binary forms, with or without
+;;; modification, are permitted provided that the following conditions
+;;; are met:
+;;; 1. Redistributions of source code must retain the above copyright
+;;;    notice, this list of conditions and the following disclaimer.
+;;; 2. Redistributions in binary form must reproduce the above copyright
+;;;    notice, this list of conditions and the following disclaimer in the
+;;;    documentation and/or other materials provided with the distribution.
+;;; 3. Neither the name of authors nor the names of its contributors
+;;;    may be used to endorse or promote products derived from this software
+;;;    without specific prior written permission.
+;;;
+;;; THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+;;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+;;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+;;; ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+;;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+;;; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+;;; OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+;;; HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+;;; LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+;;; OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+;;; SUCH DAMAGE.
+;;;;
+
 ;; SKK is a Japanese input method
 ;;
 ;; EUC-JP
@@ -31,65 +61,20 @@
 (define skk-nr-candidate-max 10)
 
 ;; key defs
-(define skk-latin-key
-  (lambda (key key-state)
-    (or
-     (generic-off-key key key-state)
-     (and (= (to-lower-char key) 108)
-	  (not (modifier-key-mask key-state))))))
-(define skk-wide-latin-key
-  (lambda (key key-state)
-    (and (= (to-lower-char key) 108)
-	 (shift-key-mask key-state))))
-(define skk-begin-conv-key
-  (lambda (key key-state)
-    (= key 32)))
-(define skk-on-key
-  (lambda (key key-state)
-    (or
-     (and (= (to-lower-char key) 106)
-	  (control-key-mask key-state))
-     (generic-on-key key key-state))))
-(define skk-hankaku-kana-key
-  (lambda (key key-state)
-    (and
-     (= (to-lower-char key) 113)
-     (control-key-mask key-state))))
-(define skk-return-key
-  (lambda (key key-state)
-    (generic-return-key key key-state)))
-(define skk-commit-key
-  (lambda (key key-state)
-    (and (= (to-lower-char key) 106)
-	 (control-key-mask key-state))))
-(define skk-next-candidate-key
-  (lambda (key key-state)
-    (generic-next-candidate-key key key-state)))
-(define skk-prev-candidate-key
-  (lambda (key key-state)
-    (or
-     (generic-prev-candidate-key key key-state)
-     (= (to-lower-char key) 120))))
-(define skk-kana-toggle-key
-  (lambda (key key-state)
-    (and
-     (= (to-lower-char key) 113)
-     (not (modifier-key-mask key-state)))))
-(define skk-cancel-key
-  (lambda (key key-state)
-    (generic-cancel-key key key-state)))
-
-(define skk-backspace-key
-  (lambda (key key-state)
-    (generic-backspace-key key key-state)))
-; 47 = '/'
-(define skk-latin-conv-key
-  (lambda (key key-state)
-    (= key 47)))
-; Q
-(define skk-kanji-mode-key
-  (lambda (key key-state)
-    (= key 81)))
+(define-key skk-latin-key? '("l" generic-off-key?))
+(define-key skk-wide-latin-key? "L")
+(define-key skk-begin-conv-key? " ")
+(define-key skk-on-key? '("<Control>j" "<Control>J" generic-on-key?))
+(define-key skk-hankaku-kana-key? '("<Control>q" "<Control>Q"))
+(define-key skk-return-key? 'generic-return-key?)
+(define-key skk-commit-key? '("<Control>j" "<Control>J"))
+(define-key skk-next-candidate-key? 'generic-next-candidate-key?)
+(define-key skk-prev-candidate-key? '("x" generic-prev-candidate-key?))
+(define-key skk-kana-toggle-key? "q")
+(define-key skk-cancel-key? 'generic-cancel-key?)
+(define-key skk-backspace-key? 'generic-backspace-key?)
+(define-key skk-latin-conv-key? "/")
+(define-key skk-kanji-mode-key? "Q")
 
 ;; style elements
 (define skk-preedit-attr-mode-mark)
@@ -591,63 +576,61 @@
 ;; called from skk-editor
 (define skk-commit-editor-context
   (lambda (sc id str)
-    (skk-flush sc)
-    (skk-context-set-child-context! sc nil)
-    (skk-commit sc id str)))
+    (let* ((psc (skk-context-parent-context sc))
+	   (okuri (skk-make-string (skk-context-okuri sc)
+				   (skk-context-kana-mode sc)))
+	   (str (if psc
+		    str
+		    (string-append str okuri))))
+      (skk-flush sc)
+      (skk-context-set-child-context! sc #f)
+      (skk-commit sc id str))))
 
+;; experimental coding style. discussions are welcome -- YamaKen
 (define skk-proc-state-direct-no-preedit
   (lambda (key key-state id sc rkc)
-    (and
-     (if (skk-cancel-key key key-state)
-	 (begin
-	   (skk-commit-raw sc id key key-state)
-	   #f)
-	 #t)
-     (if (skk-wide-latin-key key key-state)
-	 (begin
-	   (skk-context-set-state! sc 'skk-state-wide-latin)
-	   (rk-flush rkc)
-	   (skk-update-mode id sc)
-	   (skk-update-prop-label sc id)
-	   #f)
-	 #t)
-     (if (skk-latin-key key key-state)
-	 (begin
-	   (skk-context-set-state! sc 'skk-state-latin)
-	   (rk-flush rkc)
-	   (skk-update-mode id sc)
-	   (skk-update-prop-label sc id)
-	   #f)
-	 #t)
-     (if (skk-latin-conv-key key key-state)
-	 (begin
-	   (skk-context-set-state! sc 'skk-state-kanji)
-	   (skk-update-mode id sc)
-	   (skk-context-set-latin-conv! sc #t)
-	   #f)
-	 #t)
-     (if (skk-kanji-mode-key key key-state)
-	 (begin
-	   (skk-context-set-state! sc 'skk-state-kanji)
-	   (skk-update-mode id sc)
-	   (skk-context-set-latin-conv! sc #f)
-	   #f)
-	 #t)
-       (if (skk-hankaku-kana-key key key-state)
-	   (begin
-	     (skk-context-set-kana-mode! sc skk-type-hankana)
-	     (skk-update-mode id sc)
-	     (skk-update-prop-label sc id)
-	     #f)
-	   #t)
-     (if (skk-kana-toggle-key key key-state)
-	 (begin 
-	   (skk-context-kana-toggle sc)
-	   (skk-update-mode id sc)
-	   (skk-update-prop-label sc id)
-	   #f)
-	 #t)
-     #t)))
+    (cond
+     ((skk-cancel-key? key key-state)
+      (skk-commit-raw sc id key key-state)
+      #f)
+     ((skk-wide-latin-key? key key-state)
+      (skk-context-set-state! sc 'skk-state-wide-latin)
+      (rk-flush rkc)
+      (skk-update-mode id sc)
+      (skk-update-prop-label sc id)
+      #f)
+     ((skk-latin-key? key key-state)
+      (skk-context-set-state! sc 'skk-state-latin)
+      (rk-flush rkc)
+      (skk-update-mode id sc)
+      (skk-update-prop-label sc id)
+      #f)
+     ((skk-latin-conv-key? key key-state)
+      (skk-context-set-state! sc 'skk-state-kanji)
+      (skk-update-mode id sc)
+      (skk-context-set-latin-conv! sc #t)
+      #f)
+     ((skk-kanji-mode-key? key key-state)
+      (skk-context-set-state! sc 'skk-state-kanji)
+      (skk-update-mode id sc)
+      (skk-context-set-latin-conv! sc #f)
+      #f)
+     ((skk-hankaku-kana-key? key key-state)
+      (let* ((kana (skk-context-kana-mode sc))
+	     (new-kana (if (= kana skk-type-hankana)
+			   skk-type-hiragana
+			   skk-type-hankana)))
+	(skk-context-set-kana-mode! sc new-kana)
+	(skk-update-mode id sc)
+	(skk-update-prop-label sc id))
+      #f)
+     ((skk-kana-toggle-key? key key-state)
+      (skk-context-kana-toggle sc)
+      (skk-update-mode id sc)
+      (skk-update-prop-label sc id)
+      #f)
+     (else
+      #t))))
 
 (define skk-proc-state-direct
   (lambda (c key key-state)
@@ -662,12 +645,12 @@
        (if (string-equal? (rk-pending rkc) "")
 	   (skk-proc-state-direct-no-preedit key key-state id sc rkc)
 	   #t)
-       (if (skk-cancel-key key key-state)
+       (if (skk-cancel-key? key key-state)
 	   (begin
 	     (skk-flush sc)
 	     #f)
 	   #t)
-       (if (skk-backspace-key key key-state)
+       (if (skk-backspace-key? key key-state)
 	   (if (not (rk-backspace rkc))
 	       (begin
 		 (skk-commit-raw sc id key key-state)
@@ -730,17 +713,17 @@
 	   (stat (skk-context-state sc))
 	   (res))
       (and
-       (if (skk-begin-conv-key key key-state)
+       (if (skk-begin-conv-key? key key-state)
 	   (begin
 	     (skk-begin-conversion sc id)
 	     #f)
 	   #t)
-       (if (skk-cancel-key key key-state)
+       (if (skk-cancel-key? key key-state)
 	   (begin
 	     (skk-flush sc)
 	     #f)
 	   #t)
-       (if (skk-backspace-key key key-state)
+       (if (skk-backspace-key? key key-state)
 	   (begin
 	     (if (not (rk-backspace rkc))
 		 (if (> (length (skk-context-head sc)) 0)
@@ -752,8 +735,8 @@
 	     #f)
 	   #t)
        (if (or
-	    (skk-commit-key key key-state)
-	    (skk-return-key key key-state))
+	    (skk-commit-key? key key-state)
+	    (skk-return-key? key key-state))
 	   (begin
 	     (skk-commit sc id (skk-make-string
 				(skk-context-head sc)
@@ -761,7 +744,7 @@
 	     (skk-flush sc)
 	     (skk-context-set-state! sc 'skk-state-direct)
 	     (skk-update-mode id sc)
- 	     (if (skk-return-key key key-state)
+ 	     (if (skk-return-key? key key-state)
 		 (if skk-commit-newline-explicitly?
 		     (skk-commit sc id "\n")
 		     (skk-proc-state-direct c key key-state)))
@@ -788,7 +771,7 @@
 		   (skk-append-string sc tmp)))
 	     #t)
 	   #t)
-       (if (skk-kana-toggle-key key key-state)
+       (if (skk-kana-toggle-key? key key-state)
 	   (begin
 	     (if (skk-context-head sc)
 		 (skk-commit sc id (skk-make-string
@@ -838,7 +821,7 @@
 	 skk-candidate-op-count))
      (begin
        (skk-context-set-candidate-window! sc #t)
-       (im-begin-candidate
+       (im-activate-candidate-selector
 	id
 	(skk-lib-get-nr-candidates
 	 (skk-make-string (skk-context-head sc) skk-type-hiragana)
@@ -880,14 +863,14 @@
 	  (skk-check-candidate-window-begin sc id)
 	  ;;
 	  (if (skk-context-candidate-window sc)
-	      (im-update-candidate id (skk-context-nth sc)))))
+	      (im-select-candidate id (skk-context-nth sc)))))
     #f))
 
 (define skk-reset-candidate-window
   (lambda (sc id)
     (if (skk-context-candidate-window sc)
 	(begin
-	  (im-end-candidate id sc)
+	  (im-deactivate-candidate-selector id sc)
 	  (skk-context-set-candidate-window! sc #f)))
     (skk-context-set-candidate-op-count! sc 0)))
 
@@ -908,24 +891,24 @@
 	  (id (context-id c))
 	  (res ()))
       (and
-       (if (skk-next-candidate-key key key-state)
+       (if (skk-next-candidate-key? key key-state)
 	   (skk-change-candidate-index sc id #t)
 	   #t)
-       (if (skk-prev-candidate-key key key-state)
+       (if (skk-prev-candidate-key? key key-state)
 	   (skk-change-candidate-index sc id #f)
 	   #t)
-       (if (skk-cancel-key key key-state)
+       (if (skk-cancel-key? key key-state)
 	   (begin
 	     ;; back to kanji state
 	     (skk-back-to-kanji-state sc id)
 	     #f)
 	   #t)
        (if (or
-	    (skk-commit-key key key-state)
-	    (skk-return-key key key-state))
+	    (skk-commit-key? key key-state)
+	    (skk-return-key? key key-state))
 	   (begin
 	     (set! res (skk-prepare-commit-string sc id))
-	     (if (skk-return-key key key-state)
+	     (if (skk-return-key? key key-state)
 		 (begin
 		   (skk-commit sc id res)
 		   (set! res ())
@@ -965,13 +948,13 @@
 	   (id (context-id c))	   
 	   (res))
       (and
-       (if (skk-cancel-key key key-state)
+       (if (skk-cancel-key? key key-state)
 	   (begin
 	     (rk-flush rkc)
 	     (skk-context-set-state! sc 'skk-state-kanji)
 	     #f)
 	   #t)
-       (if (skk-backspace-key key key-state)
+       (if (skk-backspace-key? key key-state)
 	   (begin
 	     (if (not (rk-backspace rkc))
 		 (begin
@@ -985,8 +968,8 @@
 	   #t)
        ;; committing incomplete head: conformed the behavior to ddskk
        (if (or
-	    (skk-commit-key key key-state)
-	    (skk-return-key key key-state))
+	    (skk-commit-key? key key-state)
+	    (skk-return-key? key key-state))
 	   (begin
 	     (skk-commit sc id (skk-make-string
 				(skk-context-head sc)
@@ -994,7 +977,7 @@
 	     (skk-flush sc)
 	     (skk-context-set-state! sc 'skk-state-direct)
 	     (skk-update-mode id sc)
-	     (if (skk-return-key key key-state)
+	     (if (skk-return-key? key key-state)
 		 (skk-proc-state-direct c key key-state))
 	     #f)
 	   #t)
@@ -1017,9 +1000,10 @@
     (let ((sc (skk-current-context c))
 	  (id (context-id c)))
       (if
-       (skk-on-key key key-state)
+       (skk-on-key? key key-state)
        (begin
 	 (skk-context-set-state! sc 'skk-state-direct)
+	 (skk-context-set-kana-mode! sc skk-type-hiragana)
 	 (skk-update-mode (context-id c) sc)
 	 (skk-update-prop-label sc id))
        (skk-commit-raw sc (context-id c) key key-state))
@@ -1027,51 +1011,47 @@
 
 (define skk-proc-state-wide-latin
   (lambda (c key key-state)
-    (let* ((w (ja-wide (charcode->string key)))
+    (let* ((char (charcode->string key))
+	   (w (or (ja-direct char)
+		  (ja-wide char)))
 	   (id (context-id c))
 	   (sc (skk-current-context c)))
-      (and
-       (if (skk-on-key key key-state)
-	   (begin
-	     (skk-flush sc)
-	     (skk-update-mode id sc)
-	     (skk-update-prop-label sc id)
-	     #f)
-	   #t)
-       (if (and (modifier-key-mask key-state)
-		(not (shift-key-mask key-state)))
-	   (begin
-	     (skk-commit-raw sc id key key-state)
-	     #f)
-	   #t)
-       (if w
-	   (skk-commit sc id w)
-	   (skk-commit-raw sc id key key-state)))
+      (cond
+       ((skk-on-key? key key-state)
+	(skk-flush sc)  ; implicitly reset to 'skk-state-direct
+	(skk-context-set-kana-mode! sc skk-type-hiragana)
+	(skk-update-mode id sc)
+	(skk-update-prop-label sc id))
+       ((and (modifier-key-mask key-state)
+	     (not (shift-key-mask key-state)))
+	(skk-commit-raw sc id key key-state))
+       (w
+	(skk-commit sc id w))
+       (else
+	(skk-commit-raw sc id key key-state)))
       ())))
 
 (define skk-push-key
   (lambda (c id key key-state)
     (let* ((sc (skk-current-context c))
 	   (state (skk-context-state sc))
-	   (fun)
-	   (res))
-      (if (= state 'skk-state-direct)
-	  (set! fun skk-proc-state-direct))
-      (if (= state 'skk-state-kanji)
-	  (set! fun skk-proc-state-kanji))
-      (if (= state 'skk-state-converting)
-	  (set! fun skk-proc-state-converting))
-      (if (= state 'skk-state-okuri)
-	  (set! fun skk-proc-state-okuri))
-      (if (= state 'skk-state-latin)
-	  (set! fun skk-proc-state-latin))
-      (if (= state 'skk-state-wide-latin)
-	  (set! fun skk-proc-state-wide-latin))
-      (set! res (fun c key key-state))
+	   (fun (cond
+		 ((= state 'skk-state-direct)
+		  skk-proc-state-direct)
+		 ((= state 'skk-state-kanji)
+		  skk-proc-state-kanji)
+		 ((= state 'skk-state-converting)
+		  skk-proc-state-converting)
+		 ((= state 'skk-state-okuri)
+		  skk-proc-state-okuri)
+		 ((= state 'skk-state-latin)
+		  skk-proc-state-latin)
+		 ((= state 'skk-state-wide-latin)
+		  skk-proc-state-wide-latin)))
+	   (res (fun c key key-state)))
       (if res
 	  (skk-commit sc id res))
-      (skk-update-preedit id sc)
-      )))
+      (skk-update-preedit id sc))))
 
 (define skk-init-handler
   (lambda (id arg)
@@ -1164,27 +1144,30 @@
     (let* ((c (find-context id))
 	   (sc (context-data c)))
       (skk-flush sc)
-      (skk-update-preedit sc id)
-    (if (string-equal? message
+      (skk-update-preedit id sc)
+      (cond
+       ((string-equal? message
 		       "prop_skk_hiragana")
 	(begin
-	  (im-update-mode id 1)
-	  (skk-context-set-state! sc #t)))
-    (if (string-equal? message
+	  (skk-context-set-state! sc 'skk-state-direct)
+	  (skk-context-set-kana-mode! sc skk-type-hiragana)))
+       ((string-equal? message
 		       "prop_skk_katakana")
 	(begin
-	  (im-update-mode id 2)
-	  (skk-context-set-on! sc #t)))
-    (if (string-equal? message
+	  (skk-context-set-state! sc 'skk-state-direct)
+	  (skk-context-set-kana-mode! sc skk-type-katakana)))
+       ((string-equal? message
+		       "prop_skk_hankana")
+	(begin
+	  (skk-context-set-state! sc 'skk-state-direct)
+	  (skk-context-set-kana-mode! sc skk-type-hankana)))
+       ((string-equal? message
 		       "prop_skk_latin")
-	(begin
-	  (im-update-mode id 0)
-	  (skk-context-set-state! sc 'skk-state-latin)))
-    (if (string-equal? message
+	  (skk-context-set-state! sc 'skk-state-latin))
+       ((string-equal? message
 		       "prop_skk_wide_latin")
-	(begin
-	  (im-update-mode id 3)
 	  (skk-context-set-state! sc 'skk-state-wide-latin)))
+    (skk-update-mode id sc)
     (skk-update-prop-label sc id))))
 
 
@@ -1213,16 +1196,21 @@
 	   (state (skk-context-state sc))
 	   (kana? (skk-context-kana-mode sc))
 	   (str ""))
-      (if kana?
-	  (set! str "\tҤ餬\n")
-	  (set! str "\t\n"))
-      (if (= state 'skk-state-latin)
-	  (set! str "S\tľ\n"))
-      (if (= state 'skk-state-wide-latin)
-	  (set! str "\tѱѿ\n"))
+      (cond
+       ((= state 'skk-state-latin)
+	(set! str "S\tľ\n"))
+       ((= state 'skk-state-wide-latin)
+	(set! str "\tѱѿ\n"))
+       ((= kana skk-type-hiragana)
+	(set! str "\tҤ餬\n"))
+       ((= kana skk-type-katakana)
+	(set! str "\t\n"))
+       ((= kana skk-type-hankana)
+	(set! str "\tȾѥ\n")))
       (set! str (string-append "branch\t" str
 			       "leaf\t\tҤ餬\tҤ餬ʥ⡼\tprop_skk_hiragana\n"
 			       "leaf\t\t\tʥ⡼\tprop_skk_katakana\n"
+			       "leaf\t\tȾѥ\tȾѥʥ⡼\tprop_skk_hankana\n"
 			       "leaf\tS\tľ\tľϥ⡼\tprop_skk_latin\n"
 			       "leaf\t\tѱѿ\tѱѿ⡼\tprop_skk_wide_latin\n"))
       (im-update-prop-list id str)
