@@ -1,3 +1,33 @@
+;;;
+;;; Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+;;;
+;;; All rights reserved.
+;;;
+;;; Redistribution and use in source and binary forms, with or without
+;;; modification, are permitted provided that the following conditions
+;;; are met:
+;;; 1. Redistributions of source code must retain the above copyright
+;;;    notice, this list of conditions and the following disclaimer.
+;;; 2. Redistributions in binary form must reproduce the above copyright
+;;;    notice, this list of conditions and the following disclaimer in the
+;;;    documentation and/or other materials provided with the distribution.
+;;; 3. Neither the name of authors nor the names of its contributors
+;;;    may be used to endorse or promote products derived from this software
+;;;    without specific prior written permission.
+;;;
+;;; THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+;;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+;;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+;;; ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+;;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+;;; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+;;; OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+;;; HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+;;; LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+;;; OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+;;; SUCH DAMAGE.
+;;;;
+
 ;;uim-spellcheck is dynamic spell checker.
 
 (require "generic-key.scm")
@@ -10,12 +40,7 @@
 (define spell-always-show-window? #t)
 
 ;; key
-(define spell-on-key
-  (lambda (key key-state)
-    (or
-     (and (= (to-lower-char key) 106)
-	  (control-key-mask key-state))
-     (generic-on-key key key-state))))
+(define-key spell-on-key? '("<Control>j" "<Control>J" generic-on-key?))
 
 ;; access
 (define spell-context-immediate-commit ;;Ūspell-preedit-immididate-commit?򥪥դˤ뤫ɤ
@@ -319,21 +344,21 @@
 	   sc
 	   (spell-make-string sc (spell-context-left-string sc) #t)
 	   (spell-context-last-word sc))
-	  (im-begin-candidate
+	  (im-activate-candidate-selector
 	   id (spell-get-nr-candidates sc) 0)
 	  (spell-context-set-candidate-window! sc #t)
 	  ))
     (if (not (spell-has-preedit? sc))
 	(begin
 	  (spell-context-set-candidate-window! sc #f)
-	  (im-end-candidate id)))
+	  (im-deactivate-candidate-selector id)))
     (if (spell-context-candidate-window sc)
 	(begin
 	  (spell-get-candidates!
 	   sc
 	   (spell-make-string sc (spell-context-left-string sc) #t)
 	   (spell-context-last-word sc))	  
-	  (im-update-candidate id (spell-context-nth sc))))
+	  (im-select-candidate id (spell-context-nth sc))))
     ))
 
 (define spell-has-preedit?
@@ -352,7 +377,7 @@
 	   (direct (ja-direct (charcode->string key)))
 	   (immediate-commit))
       (and
-       (if (spell-cancel-key key key-state)
+       (if (spell-cancel-key? key key-state)
 	   (begin
 	     (spell-flush sc)
 	     #f)
@@ -363,7 +388,7 @@
 	     (spell-update-mode id sc)
 	     #f)
 	   #t)
-       (if (spell-backspace-key key key-state)
+       (if (spell-backspace-key? key key-state)
 	   (if (not (rk-backspace rkc))
 	       (begin
 		 (im-commit-raw (context-id c))
@@ -406,17 +431,17 @@
 	   (stat (spell-context-state sc))
 	   (res))
       (and
-       (if (spell-begin-conv-key key key-state)
+       (if (spell-begin-conv-key? key key-state)
 	   (begin
 	     (spell-begin-conversion sc id)
 	     #f)
 	   #t)
-       (if (spell-cancel-key key key-state)
+       (if (spell-cancel-key? key key-state)
 	   (begin
 	     (spell-flush sc)
 	     #f)
 	   #t)
-       (if (spell-backspace-key key key-state)
+       (if (spell-backspace-key? key key-state)
 	   (begin
 	     (if (not (rk-backspace rkc))
 		 (if (spell-has-preedit? sc)
@@ -428,7 +453,7 @@
 	     #f)
 	   #t)
        ;; delete
-       (if (spell-delete-key key key-state)
+       (if (spell-delete-key? key key-state)
 	   (begin
 	     (if (not (rk-delete rkc))
 		 (if (spell-context-right-string sc)
@@ -438,7 +463,7 @@
 	     #f)
 	   #t)
        ;;
-       (if (spell-commit-key key key-state)
+       (if (spell-commit-key? key key-state)
 	   (begin
 	     (im-commit id (spell-make-string
 			    sc (spell-context-left-string sc) (spell-context-kana-mode sc)))
@@ -447,7 +472,7 @@
 	     #f)
 	   #t)
        ;; left
-       (if (spell-go-left-key key key-state)
+       (if (spell-go-left-key? key key-state)
 	   (begin
 	     (if (spell-context-left-string sc)
 		 (let ((c (car (spell-context-left-string sc))))
@@ -459,7 +484,7 @@
 	     #f)
 	   #t)
        ;; right
-       (if (spell-go-right-key key key-state)
+       (if (spell-go-right-key? key key-state)
 	   (begin
 	     (if (spell-context-right-string sc)
 		 (let ((c (car (spell-context-right-string sc))))
@@ -531,12 +556,12 @@
 		 (spell-context-set-nth! sc (- (spell-get-nr-candidates sc) 1)))
 	     #f)
 	   #t)
-       (if (spell-cancel-key key key-state)
+       (if (spell-cancel-key? key key-state)
 	   (begin
 	     (spell-flush sc)
 	     #f)
 	   #t)
-       (if (spell-commit-key key key-state)
+       (if (spell-commit-key? key key-state)
 	   (begin
 	     (set! res (spell-get-current-candidate sc))
 	     (spell-commit-candidate sc)
@@ -567,7 +592,7 @@
     (let ((sc (context-data c))
 	  (id (context-id c)))
       (if
-       (spell-on-key key key-state)
+       (spell-on-key? key key-state)
        (begin
 	 (spell-context-set-mode! sc 1)
 	 (spell-update-mode id sc))
