@@ -1,3 +1,34 @@
+/*
+
+  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+
+  All rights reserved.
+
+  Redistribution and use in source and binary forms, with or without
+  modification, are permitted provided that the following conditions
+  are met:
+
+  1. Redistributions of source code must retain the above copyright
+     notice, this list of conditions and the following disclaimer.
+  2. Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+  3. Neither the name of authors nor the names of its contributors
+     may be used to endorse or promote products derived from this software
+     without specific prior written permission.
+
+  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+  SUCH DAMAGE.
+*/
 /**/
 #include <dlfcn.h>
 #include <stdlib.h>
@@ -93,6 +124,9 @@ static LISP
 init_anthy_lib()
 {
   int i;
+  if (context_slot) {
+    return siod_true_value();
+  }
   if (get_anthy_api() == -1) {
     return NIL;
   }
@@ -113,7 +147,11 @@ create_context()
   int i;
   for (i = 0; i < MAX_CONTEXT; i++) {
     if (!context_slot[i].ac) {
-      context_slot[i].ac = api.create_context();
+      struct anthy_context *ac = api.create_context();
+      if (!ac) {
+	return NIL;
+      }
+      context_slot[i].ac = ac;
       return intcons(i);
     }
   }
@@ -245,6 +283,16 @@ uim_init_anthy()
 void
 uim_quit_anthy()
 {
+  int i;
+
+  if (context_slot) {
+    for (i = 0; i < MAX_CONTEXT; i++) {
+      if(context_slot[i].ac) {
+	api.release_context(context_slot[i].ac);
+      }
+    }
+  }
+
   if (api.quit) {
     api.quit();
   }
@@ -254,4 +302,7 @@ uim_quit_anthy()
   if (api.lib) {
     dlclose(api.lib);
   }
+  if (context_slot) {
+    free(context_slot);
+  }
 }
