@@ -7,6 +7,16 @@
 
  $Id: siod.h,v 1.3 1999/09/23 23:42:37 yosh Exp $
 
+ */
+/* Notice for uim programmer:
+ *  There are two types of execution context. Some of siod 
+ *  functions can be called only from one of them.
+ *  (1) Scheme context
+ *       Current execution is under scheme stack.
+ *       Executing functions called by scheme interpreter.
+ *  (2) C context
+ *       No scheme context on the stack
+ * Yuske.
  */
 #ifndef __SIOD_H__
 #define __SIOD_H__
@@ -184,7 +194,6 @@ struct repl_hooks
 void siod_init (int argc, char **argv, int warnflag, FILE *);
 void siod_quit (void);
 
-long repl_driver (long want_init, struct repl_hooks *);
 void set_repl_hooks (void (*puts_f) (char *),
 		     LISP (*read_f) (void),
 		     LISP (*eval_f) (LISP),
@@ -192,7 +201,6 @@ void set_repl_hooks (void (*puts_f) (char *),
 LISP my_err (char *message, LISP x);
 char *get_c_string (LISP x);
 char *get_c_string_dim (LISP x, long *);
-char *try_get_c_string (LISP x);
 long get_c_long (LISP x);
 long nlength(LISP x);
 
@@ -233,27 +241,19 @@ LISP lprint (LISP exp, LISP);
 LISP lread (LISP);
 LISP lreadtk (char *, long j);
 LISP lreadf (FILE * f);
-LISP vload (char *fname, long cflag, long rflag);
 LISP require (LISP fname);
 LISP nullp (LISP x);
 LISP strcons (long length, char *data);
-LISP string_length (LISP string);
-LISP string_trim (LISP);
-LISP string_trim_left (LISP);
-LISP string_trim_right (LISP);
 //LISP copy_list_slib (LISP);
 LISP href (LISP, LISP);
 LISP hset (LISP, LISP, LISP);
 LISP equal (LISP, LISP);
 void set_fatal_exit_hook (void (*fcn) (void));
 LISP intern (LISP x);
-void init_trace (void);
 long repl_c_string (char *, long want_init, long want_print);
 LISP nreverse (LISP);
 LISP number2string (LISP, LISP, LISP, LISP);
 LISP string2number (LISP, LISP);
-LISP siod_verbose (LISP);
-int siod_verbose_check (int);
 LISP cadr (LISP);
 LISP caar (LISP);
 LISP cddr (LISP);
@@ -265,7 +265,6 @@ LISP cdaar (LISP);
 LISP cdadr (LISP);
 LISP cddar (LISP);
 LISP cdddr (LISP);
-void chk_string (LISP, char **, long *);
 LISP siod_true_value (void);
 LISP lapply (LISP fcn, LISP args);
 LISP mallocl (void *lplace, long size);
@@ -277,14 +276,6 @@ char *last_c_errmsg (int);
 LISP llast_c_errmsg (int);
 void c_provide(char *);
 
-#define SAFE_STRCPY(_to,_from) safe_strcpy((_to),sizeof(_to),(_from))
-#define SAFE_STRCAT(_to,_from) safe_strcat((_to),sizeof(_to),(_from))
-#define SAFE_STRLEN(_buff) safe_strlen((_buff),sizeof(_buff))
-
-char *safe_strcpy (char *s1, size_t size1, const char *s2);
-char *safe_strcat (char *s1, size_t size1, const char *s2);
-
-size_t safe_strlen (const char *s, size_t size);
 LISP funcall1 (LISP, LISP);
 LISP funcall2 (LISP, LISP, LISP);
 LISP apply1 (LISP, LISP, LISP);
@@ -303,21 +294,11 @@ struct gc_protected
     struct gc_protected *next;
   };
 
-#ifdef THINK_C
-extern int ipoll_counter;
-void full_interrupt_poll (int *counter);
-#define INTERRUPT_CHECK() if (--ipoll_counter < 0) full_interrupt_poll(&ipoll_counter)
-#else
 #define INTERRUPT_CHECK()
-#endif
 
 long no_interrupt (long n);
 void fput_st (FILE * f, char *st);
 void put_st (char *st);
 void gc_for_newcell (void);
-void gc_ms_stats_start (void);
-void gc_ms_stats_end (void);
-
-void init_subrs_a (void);
 
 #endif /* __SIOD_H__ */
