@@ -73,8 +73,10 @@
   removed vms,array,thinkc functions (Oct-03) Yusuke TABATA
   removed copygc (Nov-03) Yusuke TABATA
   removed many unneeded functionality (03-04) Yusuke TABATA
+  removed sliba.c (Feb-04) Yusuke TABATA
  */
 #include <stdio.h>
+#include <stdarg.h>
 #include <string.h>
 #include <ctype.h>
 #include <setjmp.h>
@@ -423,7 +425,193 @@ repl (struct repl_hooks *h)
   return (0);
 }
 
-long
+static LISP
+setcdr (LISP cell, LISP value)
+{
+  if NCONSP
+    (cell) my_err ("wta to setcdr", cell);
+  return (CDR (cell) = value);
+}
+
+LISP
+newcell (long type)
+{
+  LISP z;
+  NEWCELL (z, type);
+  return (z);
+}
+
+static LISP
+fopen_cg (FILE * (*fcn) (const char *, const char *), char *name, char *how)
+{
+  LISP sym;
+  long flag;
+  char errmsg[80];
+  flag = no_interrupt (1);
+  sym = newcell (tc_c_file);
+  sym->storage_as.c_file.f = (FILE *) NULL;
+  sym->storage_as.c_file.name = (char *) NULL;
+  if (!(sym->storage_as.c_file.f = (*fcn) (name, how)))
+    {
+      snprintf(errmsg, 79, "could not open %s", name);
+      my_err (errmsg, llast_c_errmsg (-1));
+    }
+  sym->storage_as.c_file.name = (char *) must_malloc (strlen (name) + 1);
+  strcpy (sym->storage_as.c_file.name, name);
+  no_interrupt (flag);
+  return (sym);
+}
+
+static LISP
+fopen_c (char *name, char *how)
+{
+  return (fopen_cg (fopen, name, how));
+}
+
+static LISP
+fopen_l (LISP name, LISP how)
+{
+  return (fopen_c (get_c_string (name), NULLP (how) ? "r" : get_c_string (how)));
+}
+
+static void
+file_gc_free (LISP ptr)
+{
+  if (ptr->storage_as.c_file.f)
+    {
+      fclose (ptr->storage_as.c_file.f);
+      ptr->storage_as.c_file.f = (FILE *) NULL;
+    }
+  if (ptr->storage_as.c_file.name)
+    {
+      free (ptr->storage_as.c_file.name);
+      ptr->storage_as.c_file.name = NULL;
+    }
+}
+
+static LISP
+fclose_l (LISP p)
+{
+  long flag;
+  flag = no_interrupt (1);
+  if NTYPEP
+    (p, tc_c_file) my_err ("not a file", p);
+  file_gc_free (p);
+  no_interrupt (flag);
+  return (NIL);
+}
+
+static LISP
+lprin1 (LISP exp, LISP lf)
+{
+  FILE *f = get_c_file (lf, siod_output);
+  lprin1f (exp, f);
+  return (NIL);
+}
+
+static LISP
+vload (char *fname, long cflag, long rflag)
+{
+  LISP form, result, tail, lf, reader = NIL;
+  FILE *f;
+  int c, j;
+  char buffer[512];
+  char *fnbuf;
+  char  *key = "parser:", *start, *end, *ftype = ".scm";
+  if (rflag)
+    {
+      int iflag;
+      iflag = no_interrupt (1);
+      /*      if ((f = fopen (fname, "r")))
+	fclose (f);
+	else*/
+      if ((fname[0] != '/'))
+	{
+	  fnbuf = alloca(strlen(siod_lib) + strlen(fname) + 2);
+	  strcpy (fnbuf, siod_lib);
+	  strcat (fnbuf, "/");
+	  strcat (fnbuf, fname);
+	  if ((f = fopen (fnbuf, "r")))
+	    {
+	      fname = fnbuf;
+	      fclose (f);
+	    }
+	}
+      no_interrupt (iflag);
+    }
+  if (siod_verbose_level >= 3)
+    {
+      put_st ("loading ");
+      put_st (fname);
+      put_st ("\n");
+    }
+  lf = fopen_c (fname, "r");
+  f = lf->storage_as.c_file.f;
+  result = NIL;
+  tail = NIL;
+  j = 0;
+  buffer[0] = 0;
+  c = getc (f);
+  while ((c == '#') || (c == ';'))
+    {
+      while (((c = getc (f)) != EOF) && (c != '\n'))
+	if ((j + 1) < sizeof (buffer))
+	  {
+	    buffer[j] = c;
+	    buffer[++j] = 0;
+	  }
+      if (c == '\n')
+	c = getc (f);
+    }
+  if (c != EOF)
+    ungetc (c, f);
+  if ((start = strstr (buffer, key)))
+    {
+      for (end = &start[strlen (key)];
+	   *end && isalnum (*end);
+	   ++end);
+      j = end - start;
+      memmove (buffer, start, j);
+      buffer[strlen (key) - 1] = '_';
+      buffer[j] = 0;
+      strcat (buffer, ftype);
+      require (strcons (-1, buffer));
+      buffer[j] = 0;
+      reader = rintern (buffer);
+      reader = funcall1 (leval (reader, NIL), reader);
+      if (siod_verbose_level >= 5)
+	{
+	  put_st ("parser:");
+	  lprin1 (reader, NIL);
+	  put_st ("\n");
+	}
+    }
+  while (1)
+    {
+      form = NULLP (reader) ? lread (lf) : funcall1 (reader, lf);
+      if EQ
+	(form, eof_val) break;
+      if (siod_verbose_level >= 5)
+	lprint (form, NIL);
+      if (cflag)
+	{
+	  form = cons (form, NIL);
+	  if NULLP
+	    (result)
+	      result = tail = form;
+	  else
+	    tail = setcdr (tail, form);
+	}
+      else
+	leval (form, NIL);
+    }
+  fclose_l (lf);
+  if (siod_verbose_level >= 3)
+    put_st ("done.\n");
+  return (result);
+}
+
+static long
 repl_driver (long want_init, struct repl_hooks *h)
 {
   int k;
@@ -717,26 +905,42 @@ lreadr (struct gen_readio *f)
   return (my_err ("token larger than TKBUFFERN", NIL));
 }
 
+/* Iterative version */
 static LISP
 lreadparen (struct gen_readio * f)
 {
   int c;
-  LISP tmp;
-  c = flush_ws (f, "end of file inside list");
-  if (c == ')')
-    return (NIL);
-  UNGETC_FCN (c, f);
-  tmp = lreadr (f);
-  if EQ
-    (tmp, sym_dot)
+  LISP tmp, l = NIL;
+  LISP last = l;
+
+  while ((c = flush_ws(f, "end of file inside list")) != ')')
     {
+      UNGETC_FCN (c,f);
       tmp = lreadr (f);
-      c = flush_ws (f, "end of file inside list");
-      if (c != ')')
-	my_err ("missing close paren", NIL);
-      return (tmp);
+      if EQ
+	(tmp, sym_dot)
+	{
+	  tmp = lreadr (f);
+	  c = flush_ws (f, "end of file inside list");
+	  if (c != ')')
+	    my_err ("missing close paren", NIL);
+	  if (l == NIL)
+	    my_err("nor car for dotted pair", NIL);
+	  CDR (last) = tmp;
+	  break;
+	}
+      if (l == NIL)
+	{
+	  l = cons (tmp, NIL);
+	  last = l;
+	}
+      else
+	{
+	  CDR (last) = cons (tmp, NIL);
+	  last = cdr (last);
+	}
     }
-  return (cons (tmp, lreadparen (f)));
+  return l;
 }
 
 static LISP
@@ -868,6 +1072,20 @@ set_fatal_exit_hook (void (*fcn) (void))
 
 static long inside_err = 0;
 
+
+static char *
+try_get_c_string (LISP x)
+{
+  if TYPEP
+    (x, tc_symbol)
+      return (PNAME (x));
+  else if TYPEP
+    (x, tc_string)
+      return (x->storage_as.string.data);
+  else
+    return (NULL);
+}
+
 LISP
 my_err (char *message, LISP x)
 {
@@ -937,14 +1155,6 @@ my_err (char *message, LISP x)
 }
 
 
-static LISP
-setcdr (LISP cell, LISP value)
-{
-  if NCONSP
-    (cell) my_err ("wta to setcdr", cell);
-  return (CDR (cell) = value);
-}
-
 static LISP
 last (LISP l)
 {
@@ -991,19 +1201,6 @@ stack_limit (LISP amount, LISP silent)
     return (intcons (stack_size));
 }
 
-char *
-try_get_c_string (LISP x)
-{
-  if TYPEP
-    (x, tc_symbol)
-      return (PNAME (x));
-  else if TYPEP
-    (x, tc_string)
-      return (x->storage_as.string.data);
-  else
-    return (NULL);
-}
-
 char *
 get_c_string (LISP x)
 {
@@ -1051,14 +1248,6 @@ gc_fatal_error (void)
   my_err ("ran out of storage", NIL);
 }
 
-LISP
-newcell (long type)
-{
-  LISP z;
-  NEWCELL (z, type);
-  return (z);
-}
-
 LISP
 cons (LISP x, LISP y)
 {
@@ -1775,21 +1964,6 @@ init_storage_1 (void)
     }
 }
 
-static void
-file_gc_free (LISP ptr)
-{
-  if (ptr->storage_as.c_file.f)
-    {
-      fclose (ptr->storage_as.c_file.f);
-      ptr->storage_as.c_file.f = (FILE *) NULL;
-    }
-  if (ptr->storage_as.c_file.name)
-    {
-      free (ptr->storage_as.c_file.name);
-      ptr->storage_as.c_file.name = NULL;
-    }
-}
-
 static void
 init_storage (void)
 {
@@ -2093,6 +2267,26 @@ mark_protected_registers (void)
     }
 }
 
+static void
+gc_ms_stats_start (void)
+{
+  gc_rt = myruntime ();
+  gc_cells_collected = 0;
+  if (gc_status_flag && (siod_verbose_level >= 4))
+    fprintf (siod_output, "[starting GC]\n");
+}
+
+static void
+gc_ms_stats_end (void)
+{
+  gc_rt = myruntime () - gc_rt;
+  gc_time_taken = gc_time_taken + gc_rt;
+  if (gc_status_flag && (siod_verbose_level >= 4))
+    fprintf (siod_output, "[GC took %g cpu seconds, %ld cells collected]\n",
+	     gc_rt,
+	     gc_cells_collected);
+}
+
 static void
 gc_mark_and_sweep (void)
 {
@@ -2147,27 +2341,6 @@ gc_for_newcell (void)
     allocate_aheap ();
 }
 
-
-void
-gc_ms_stats_start (void)
-{
-  gc_rt = myruntime ();
-  gc_cells_collected = 0;
-  if (gc_status_flag && (siod_verbose_level >= 4))
-    fprintf (siod_output, "[starting GC]\n");
-}
-
-void
-gc_ms_stats_end (void)
-{
-  gc_rt = myruntime () - gc_rt;
-  gc_time_taken = gc_time_taken + gc_rt;
-  if (gc_status_flag && (siod_verbose_level >= 4))
-    fprintf (siod_output, "[GC took %g cpu seconds, %ld cells collected]\n",
-	     gc_rt,
-	     gc_cells_collected);
-}
-
 static LISP
 user_gc (LISP args)
 {
@@ -2855,7 +3028,7 @@ symbolconc (LISP args)
   return (rintern (tkbuffer));
 }
 
-char *
+static char *
 subr_kind_str (long n)
 {
   switch (n)
@@ -2991,7 +3164,7 @@ err_wta_str (LISP exp)
 }
 
 
-LISP
+static LISP
 string_length (LISP string)
 {
   if NTYPEP
@@ -3043,14 +3216,6 @@ flprint (LISP exp, LISP lf)
   return (NIL);
 }
 
-LISP
-lprin1 (LISP exp, LISP lf)
-{
-  FILE *f = get_c_file (lf, siod_output);
-  lprin1f (exp, f);
-  return (NIL);
-}
-
 LISP
 lprin1f (LISP exp, FILE * f)
 {
@@ -3185,40 +3350,6 @@ apropos (LISP matchl)
   return (result);
 }
 
-static LISP
-fopen_cg (FILE * (*fcn) (const char *, const char *), char *name, char *how)
-{
-  LISP sym;
-  long flag;
-  char errmsg[80];
-  flag = no_interrupt (1);
-  sym = newcell (tc_c_file);
-  sym->storage_as.c_file.f = (FILE *) NULL;
-  sym->storage_as.c_file.name = (char *) NULL;
-  if (!(sym->storage_as.c_file.f = (*fcn) (name, how)))
-    {
-      SAFE_STRCPY (errmsg, "could not open ");
-      SAFE_STRCAT (errmsg, name);
-      my_err (errmsg, llast_c_errmsg (-1));
-    }
-  sym->storage_as.c_file.name = (char *) must_malloc (strlen (name) + 1);
-  strcpy (sym->storage_as.c_file.name, name);
-  no_interrupt (flag);
-  return (sym);
-}
-
-static LISP
-fopen_c (char *name, char *how)
-{
-  return (fopen_cg (fopen, name, how));
-}
-
-static LISP
-fopen_l (LISP name, LISP how)
-{
-  return (fopen_c (get_c_string (name), NULLP (how) ? "r" : get_c_string (how)));
-}
-
 LISP
 delq (LISP elem, LISP l)
 {
@@ -3231,18 +3362,6 @@ delq (LISP elem, LISP l)
   return (l);
 }
 
-static LISP
-fclose_l (LISP p)
-{
-  long flag;
-  flag = no_interrupt (1);
-  if NTYPEP
-    (p, tc_c_file) my_err ("not a file", p);
-  file_gc_free (p);
-  no_interrupt (flag);
-  return (NIL);
-}
-
 
 static LISP
 memq (LISP x, LISP il)
@@ -3285,108 +3404,6 @@ c_provide(char *name)
   provide(rintern(name));
 }
 
-LISP
-vload (char *fname, long cflag, long rflag)
-{
-  LISP form, result, tail, lf, reader = NIL;
-  FILE *f;
-  int c, j;
-  char buffer[512];
-  char *fnbuf;
-  char  *key = "parser:", *start, *end, *ftype = ".scm";
-  if (rflag)
-    {
-      int iflag;
-      iflag = no_interrupt (1);
-      /*      if ((f = fopen (fname, "r")))
-	fclose (f);
-	else*/
-      if ((fname[0] != '/'))
-	{
-	  fnbuf = alloca(strlen(siod_lib) + strlen(fname) + 2);
-	  strcpy (fnbuf, siod_lib);
-	  strcat (fnbuf, "/");
-	  strcat (fnbuf, fname);
-	  if ((f = fopen (fnbuf, "r")))
-	    {
-	      fname = fnbuf;
-	      fclose (f);
-	    }
-	}
-      no_interrupt (iflag);
-    }
-  if (siod_verbose_level >= 3)
-    {
-      put_st ("loading ");
-      put_st (fname);
-      put_st ("\n");
-    }
-  lf = fopen_c (fname, "r");
-  f = lf->storage_as.c_file.f;
-  result = NIL;
-  tail = NIL;
-  j = 0;
-  buffer[0] = 0;
-  c = getc (f);
-  while ((c == '#') || (c == ';'))
-    {
-      while (((c = getc (f)) != EOF) && (c != '\n'))
-	if ((j + 1) < sizeof (buffer))
-	  {
-	    buffer[j] = c;
-	    buffer[++j] = 0;
-	  }
-      if (c == '\n')
-	c = getc (f);
-    }
-  if (c != EOF)
-    ungetc (c, f);
-  if ((start = strstr (buffer, key)))
-    {
-      for (end = &start[strlen (key)];
-	   *end && isalnum (*end);
-	   ++end);
-      j = end - start;
-      memmove (buffer, start, j);
-      buffer[strlen (key) - 1] = '_';
-      buffer[j] = 0;
-      strcat (buffer, ftype);
-      require (strcons (-1, buffer));
-      buffer[j] = 0;
-      reader = rintern (buffer);
-      reader = funcall1 (leval (reader, NIL), reader);
-      if (siod_verbose_level >= 5)
-	{
-	  put_st ("parser:");
-	  lprin1 (reader, NIL);
-	  put_st ("\n");
-	}
-    }
-  while (1)
-    {
-      form = NULLP (reader) ? lread (lf) : funcall1 (reader, lf);
-      if EQ
-	(form, eof_val) break;
-      if (siod_verbose_level >= 5)
-	lprint (form, NIL);
-      if (cflag)
-	{
-	  form = cons (form, NIL);
-	  if NULLP
-	    (result)
-	      result = tail = form;
-	  else
-	    tail = setcdr (tail, form);
-	}
-      else
-	leval (form, NIL);
-    }
-  fclose_l (lf);
-  if (siod_verbose_level >= 3)
-    put_st ("done.\n");
-  return (result);
-}
-
 static LISP
 load (LISP fname, LISP cflag, LISP rflag)
 {
@@ -3539,7 +3556,7 @@ nreverse (LISP x)
   return (newp);
 }
 
-LISP
+static LISP
 siod_verbose (LISP arg)
 {
   if NNULLP
@@ -3548,12 +3565,6 @@ siod_verbose (LISP arg)
   return (intcons (siod_verbose_level));
 }
 
-int
-siod_verbose_check (int level)
-{
-  return ((siod_verbose_level >= level) ? 1 : 0);
-}
-
 LISP
 lruntime (void)
 {
@@ -3661,46 +3672,6 @@ lllast_c_errmsg (void)
   return (llast_c_errmsg (-1));
 }
 
-size_t
-safe_strlen (const char *s, size_t size)
-{
-  char *end;
-  if ((end = (char *) memchr (s, 0, size)))
-    return (end - s);
-  else
-    return (size);
-}
-
-char *
-safe_strcpy (char *s1, size_t size1, const char *s2)
-{
-  size_t len2;
-  if (size1 == 0)
-    return (s1);
-  len2 = strlen (s2);
-  if (len2 < size1)
-    {
-      if (len2)
-	memcpy (s1, s2, len2);
-      s1[len2] = 0;
-    }
-  else
-    {
-      memcpy (s1, s2, size1);
-      s1[size1 - 1] = 0;
-    }
-  return (s1);
-}
-
-char *
-safe_strcat (char *s1, size_t size1, const char *s2)
-{
-  size_t len1;
-  len1 = safe_strlen (s1, size1);
-  safe_strcpy (&s1[len1], size1 - len1, s2);
-  return (s1);
-}
-
 static LISP
 parser_read (LISP ignore)
 {
@@ -3957,6 +3928,134 @@ funcall2 (LISP fcn, LISP a1, LISP a2)
 }
 
 
+static LISP
+assv (LISP x, LISP alist)
+{
+  LISP l, tmp;
+  for (l = alist; CONSP (l); l = CDR (l))
+    {
+      tmp = CAR (l);
+      if (CONSP (tmp) && NNULLP (eql (CAR (tmp), x)))
+	return (tmp);
+      INTERRUPT_CHECK ();
+    }
+  if EQ
+    (l, NIL) return (NIL);
+  return (my_err ("improper list to assv", alist));
+}
+
+static LISP
+lstrcmp (LISP s1, LISP s2)
+{
+  return (intcons (strcmp (get_c_string (s1), get_c_string (s2))));
+}
+
+static LISP
+member (LISP x, LISP il)
+{
+  LISP l, tmp;
+  for (l = il; CONSP (l); l = CDR (l))
+    {
+      tmp = CAR (l);
+      if NNULLP
+	(equal (x, tmp)) return (l);
+      INTERRUPT_CHECK ();
+    }
+  if EQ
+    (l, NIL) return (NIL);
+  return (my_err ("improper list to member", il));
+}
+
+static LISP
+memv (LISP x, LISP il)
+{
+  LISP l, tmp;
+  for (l = il; CONSP (l); l = CDR (l))
+    {
+      tmp = CAR (l);
+      if NNULLP
+	(eql (x, tmp)) return (l);
+      INTERRUPT_CHECK ();
+    }
+  if EQ
+    (l, NIL) return (NIL);
+  return (my_err ("improper list to memv", il));
+}
+
+LISP
+lsubset (LISP fcn, LISP l)
+{
+  LISP result = NIL, v;
+  for (v = l; CONSP (v); v = CDR (v))
+    if NNULLP
+      (funcall1 (fcn, CAR (v)))
+	result = cons (CAR (v), result);
+  return (nreverse (result));
+}
+
+LISP
+listn (long n,...)
+{
+  LISP result, ptr;
+  long j;
+  va_list args;
+  for (j = 0, result = NIL; j < n; ++j)
+    result = cons (NIL, result);
+  va_start (args, n);
+  for (j = 0, ptr = result; j < n; ptr = cdr (ptr), ++j)
+    setcar (ptr, va_arg (args, LISP));
+  va_end (args);
+  return (result);
+}
+
+static LISP
+ltypeof (LISP obj)
+{
+  long x;
+  x = TYPE (obj);
+  switch (x)
+    {
+    case tc_nil:
+      return (rintern ("tc_nil"));
+    case tc_cons:
+      return (rintern ("tc_cons"));
+    case tc_intnum:
+      return (rintern ("tc_intnum"));
+    case tc_symbol:
+      return (rintern ("tc_symbol"));
+    case tc_subr_0:
+      return (rintern ("tc_subr_0"));
+    case tc_subr_1:
+      return (rintern ("tc_subr_1"));
+    case tc_subr_2:
+      return (rintern ("tc_subr_2"));
+    case tc_subr_2n:
+      return (rintern ("tc_subr_2n"));
+    case tc_subr_3:
+      return (rintern ("tc_subr_3"));
+    case tc_subr_4:
+      return (rintern ("tc_subr_4"));
+    case tc_subr_5:
+      return (rintern ("tc_subr_5"));
+    case tc_lsubr:
+      return (rintern ("tc_lsubr"));
+    case tc_fsubr:
+      return (rintern ("tc_fsubr"));
+    case tc_msubr:
+      return (rintern ("tc_msubr"));
+    case tc_closure:
+      return (rintern ("tc_closure"));
+    case tc_free_cell:
+      return (rintern ("tc_free_cell"));
+    case tc_string:
+      return (rintern ("tc_string"));
+    case tc_c_file:
+      return (rintern ("tc_c_file"));
+    default:
+      return (intcons (x));
+    }
+}
+
 static void
 init_subrs (void)
 {
@@ -4082,7 +4181,12 @@ init_subrs (void)
   init_subr_2 ("nth", nth);
   init_subr_1 ("butlast", butlast);
 
-  init_subrs_a ();
+  init_subr_2 ("assv", assv);
+  init_subr_2 ("strcmp", lstrcmp);
+  init_subr_2 ("subset", lsubset);
+  init_subr_1 ("typeof", ltypeof);
+  init_subr_2 ("memv", memv);
+  init_subr_2 ("member", member);
   init_slib_version ();
 }
 
