@@ -1,4 +1,38 @@
+/*
+
+  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+
+  All rights reserved.
+
+  Redistribution and use in source and binary forms, with or without
+  modification, are permitted provided that the following conditions
+  are met:
+
+  1. Redistributions of source code must retain the above copyright
+     notice, this list of conditions and the following disclaimer.
+  2. Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+  3. Neither the name of authors nor the names of its contributors
+     may be used to endorse or promote products derived from this software
+     without specific prior written permission.
+
+  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+  SUCH DAMAGE.
+
+*/
+
 #include <ctype.h>
+#include <iconv.h>
 #include <string.h>
 #include <stdlib.h>
 #include "context.h"
@@ -35,10 +69,10 @@ uim_flush_cb(uim_context uc)
 	free(s);
       }
       break;
-    case CAND_BEGIN_CB:
+    case CAND_ACTIVATE_CB:
       {
-	if (uc->candidate_selector_begin_cb) {
-	  uc->candidate_selector_begin_cb(uc->ptr, tmp->n1, tmp->n2);
+	if (uc->candidate_selector_activate_cb) {
+	  uc->candidate_selector_activate_cb(uc->ptr, tmp->n1, tmp->n2);
 	}
       }
       break;
@@ -56,10 +90,10 @@ uim_flush_cb(uim_context uc)
 	}
       }
       break;
-    case CAND_END_CB:
+    case CAND_DEACTIVATE_CB:
       {
-	if (uc->candidate_selector_end_cb) {
-	  uc->candidate_selector_end_cb(uc->ptr);
+	if (uc->candidate_selector_deactivate_cb) {
+	  uc->candidate_selector_deactivate_cb(uc->ptr);
 	}
       }
       break;
@@ -160,11 +194,11 @@ char *
 uim_iconv_code_conv(void *obj, const char *str)
 {
   iconv_t ic;
-  int len;
-  int buflen;
+  size_t len;
+  size_t buflen;
   char *realbuf;
   char *outbuf;
-  const char *inbuf;
+  char *inbuf;
 
   ic = (iconv_t)obj;
   if(!str)
@@ -173,7 +207,7 @@ uim_iconv_code_conv(void *obj, const char *str)
   buflen = (len * 6)+3;
   realbuf = alloca(buflen);
   outbuf = realbuf;
-  inbuf = str;
+  inbuf = (char *) str;
   if (!ic) {
     return strdup(str);
   }
@@ -285,14 +319,14 @@ im_get_raw_key_str(LISP key_, LISP key_state_)
   } else {
     return NIL;
   }
-  if ((key_state != 0 && key_state != UKey_Shift) ||
+  if ((key_state != 0 && key_state != UMod_Shift) ||
       key > 255) {
     return NIL;
   }
   
   buf[0] = key;
   buf[1] = 0;
-  if (key_state == UKey_Shift) {
+  if (key_state == UMod_Shift) {
     buf[0] = toupper(buf[0]);
   }
   return strcons(1, buf);
@@ -468,13 +502,13 @@ im_register_im(LISP name, LISP lang, LISP enc)
 }
 
 static LISP
-im_begin_candidate(LISP id_, LISP nr_, LISP display_limit_)
+im_activate_candidate_selector(LISP id_, LISP nr_, LISP display_limit_)
 {
   int id = get_c_long(id_);
   int display_limit = get_c_long(display_limit_);
   int nr = get_c_long(nr_);
   uim_context uc = uim_find_context(id);
-  uim_schedule_cb(uc, CAND_BEGIN_CB, NULL, nr, display_limit);
+  uim_schedule_cb(uc, CAND_ACTIVATE_CB, NULL, nr, display_limit);
   return NIL;
 }
 
@@ -508,11 +542,11 @@ im_shift_page_candidate(LISP id_, LISP dir_)
 }
 
 static LISP
-im_end_candidate(LISP id_)
+im_deactivate_candidate_selector(LISP id_)
 {
   int id = get_c_long(id_);
   uim_context uc = uim_find_context(id);
-  uim_schedule_cb(uc, CAND_END_CB, NULL, 0, 0);
+  uim_schedule_cb(uc, CAND_DEACTIVATE_CB, NULL, 0, 0);
   return NIL;
 }
 
@@ -585,8 +619,8 @@ uim_init_im_subrs()
   /**/
   init_subr_2("im-update-mode", im_update_mode);
   /**/
-  init_subr_3("im-begin-candidate",  im_begin_candidate);
-  init_subr_2("im-update-candidate", im_select_candidate);
+  init_subr_3("im-activate-candidate-selector",  im_activate_candidate_selector);
+  init_subr_2("im-select-candidate", im_select_candidate);
   init_subr_2("im-shift-page-candidate", im_shift_page_candidate);
-  init_subr_1("im-end-candidate",    im_end_candidate);
+  init_subr_1("im-deactivate-candidate-selector", im_deactivate_candidate_selector);
 }
