@@ -1,3 +1,36 @@
+/*
+
+  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+
+  All rights reserved.
+
+  Redistribution and use in source and binary forms, with or without
+  modification, are permitted provided that the following conditions
+  are met:
+
+  1. Redistributions of source code must retain the above copyright
+     notice, this list of conditions and the following disclaimer.
+  2. Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+  3. Neither the name of authors nor the names of its contributors
+     may be used to endorse or promote products derived from this software
+     without specific prior written permission.
+
+  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+  SUCH DAMAGE.
+
+*/
+
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -16,7 +49,7 @@ static void (*uim_disconnect_cb)(void);
 
 static void uim_helper_client_focus(uim_context uc, int flg);
 
-#define BUFFER_SIZE 1024
+#define BUFFER_SIZE 2048
 /*Common buffer for some functions's temporary buffer.
   Pay attention for use.*/
 static char uim_help_buf[BUFFER_SIZE];
@@ -33,7 +66,7 @@ get_server_command(void)
 
 int uim_helper_init_client_fd(void (*disconnect_cb)(void))
 {
-  struct stat lst, fst;
+  struct stat lst;
   int fd;
   struct sockaddr_un server;
   char *path = uim_helper_get_pathname();
@@ -55,15 +88,15 @@ int uim_helper_init_client_fd(void (*disconnect_cb)(void))
     return -1;
   }
   
-  if (fstat(fd, &fst) != 0) {
-    close(fd);
-    return -1;
-  }  
-  if (fst.st_uid != getuid()) {
-    close(fd);
-    return -1;
+#ifdef LOCAL_CREDS /* for NetBSD */
+  /* Set the socket to receive credentials on the next message */
+  {
+    int on = 1;
+    setsockopt(fd, 0, LOCAL_CREDS, &on, sizeof(on));
   }
-  
+#endif
+
+
   if(connect(fd, (struct sockaddr *)&server,sizeof(server)) == -1){
     int serv_pid = 0;
     FILE *serv_r = NULL, *serv_w = NULL;
@@ -85,6 +118,11 @@ int uim_helper_init_client_fd(void (*disconnect_cb)(void))
     }
   }
 
+  if (uim_helper_check_connection_fd(fd)) {
+    close(fd);
+    return -1;
+  }
+ 
   uim_disconnect_cb = disconnect_cb;
   uim_read_buf_count = 0;
   uim_read_buf_error = 0;
@@ -146,10 +184,15 @@ uim_helper_client_get_prop_list(void)
 void
 uim_helper_read_proc(int fd)
 {
-  int rc;  
+  int rc, count;
 
-  rc = read(fd, uim_read_buf, BUFFER_SIZE - uim_read_buf_count);
+  count = BUFFER_SIZE - uim_read_buf_count;
+  if (count == 0) {
+    return ;
+  }
+  rc = read(fd, uim_read_buf, count);
   if (rc == 0) {
+    close(fd);
     if (uim_disconnect_cb) {
       uim_disconnect_cb();
     }
@@ -172,6 +215,7 @@ uim_helper_get_message()
 {
   int i;
   char buf[BUFFER_SIZE];
+
   for (i = 0; i < uim_read_buf_count - 1; i++) {
     if (uim_read_buf[i] == '\n' &&
 	uim_read_buf[i+1] == '\n') {
@@ -185,7 +229,8 @@ uim_helper_get_message()
       return strdup(buf);
     }
   }
-  if (uim_read_buf_count) {
+  if (uim_read_buf_count == BUFFER_SIZE) {
+    uim_read_buf_count = 0;
     uim_read_buf_error = 1;
   }
   return NULL;
