@@ -1,3 +1,36 @@
+/*
+
+  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+
+  All rights reserved.
+
+  Redistribution and use in source and binary forms, with or without
+  modification, are permitted provided that the following conditions
+  are met:
+
+  1. Redistributions of source code must retain the above copyright
+     notice, this list of conditions and the following disclaimer.
+  2. Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+  3. Neither the name of authors nor the names of its contributors
+     may be used to endorse or promote products derived from this software
+     without specific prior written permission.
+
+  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+  SUCH DAMAGE.
+
+*/
+
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -104,12 +137,12 @@ parse_content(char *content, struct client *cl)
 {
   int i;
 
-    for (i = 0; i < MAX_CLIENT; i ++){
-      if(clients[i].fd != -1 && 
-	 clients[i].fd != cl->fd && 
-	 uim_helpler_fd_writable(clients[i].fd))
-	write(clients[i].fd, content, strlen(content));
-    }
+  for (i = 0; i < MAX_CLIENT; i ++){
+    if(clients[i].fd != -1 && 
+       clients[i].fd != cl->fd && 
+       uim_helpler_fd_writable(clients[i].fd))
+      write(clients[i].fd, content, strlen(content));
+  }
 
     
     //    write(cl->fd, buf, strlen(buf));
@@ -122,8 +155,12 @@ int proc_func(struct client *cl)
 
   /* do read */
   rc = read(cl->fd, buf, BUFFER_SIZE - 1);
+  if (rc == 0) {
+    close(cl->fd);
+    return -1;
+  }
 
-  if (rc <= 0) {
+  if (rc < 0) {
     return -1;
   }
 
@@ -143,8 +180,7 @@ int proc_func(struct client *cl)
 }
 
 void
-uim_helper_server_process_connection(int serv_fd,
-				     int (*proc_func)(struct client * cl) )
+uim_helper_server_process_connection(int serv_fd)
 {
   int i;
   int fd_biggest = 0;
@@ -169,10 +205,10 @@ uim_helper_server_process_connection(int serv_fd,
     }
 
     /* call select() */
-    if ( select(fd_biggest+1, &readfds, NULL, NULL, NULL) < 0 ) {
+    if (select(fd_biggest+1, &readfds, NULL, NULL, NULL) < 0) {
       perror("select faild");
     }
-    
+
     /* for accept new connection */
     if (FD_ISSET(serv_fd, &readfds)) {
       struct sockaddr_un clientsoc;
@@ -192,18 +228,21 @@ uim_helper_server_process_connection(int serv_fd,
 	continue;
       }
       cl->fd = new_fd;
-      /*write(cl->fd, "OK\n", 3);*/
-
-      /*      fprintf(stderr,"accept new fd:%d\n",new_fd);*/
+#ifdef LOCAL_CREDS	/* for NetBSD */
+      {
+	char buf[1] = { '\0' };
+	write(cl->fd, buf, 1);
+      }
+#endif
     }    
     
     /* check data from clients reached */
-    for (i = 0; i < MAX_CLIENT; i ++){
+    for (i = 0; i < MAX_CLIENT; i ++) {
       if (clients[i].fd != -1 &&
 	  FD_ISSET(clients[i].fd, &readfds)) {
 	int result;
 	/* actual process */
-	result = (*proc_func)(&clients[i]);
+	result = proc_func(&clients[i]);
 
 	if (result < 0) {
 	  close(clients[i].fd);
@@ -237,7 +276,7 @@ main(int argc, char **argv)
 
   free(path);
 
-  uim_helper_server_process_connection(serv_fd, proc_func);
+  uim_helper_server_process_connection(serv_fd);
 
   return 0;
 }
