@@ -1,3 +1,36 @@
+/*
+
+  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+
+  All rights reserved.
+
+  Redistribution and use in source and binary forms, with or without
+  modification, are permitted provided that the following conditions
+  are met:
+
+  1. Redistributions of source code must retain the above copyright
+     notice, this list of conditions and the following disclaimer.
+  2. Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+  3. Neither the name of authors nor the names of its contributors
+     may be used to endorse or promote products derived from this software
+     without specific prior written permission.
+
+  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+  SUCH DAMAGE.
+
+*/
+
 #include <stdio.h>
 #include "context.h"
 
@@ -92,6 +125,8 @@ static struct {
   {UKey_Alt_key, "Alt_key"},
   {UKey_Control_key, "Control_key"},
   {UKey_Meta_key, "Meta_key"},
+  {UKey_Super_key, "Super_key"},
+  {UKey_Hyper_key, "Hyper_key"},
   /*  {UKey_Other, "other"},*/
   {0,0}
 };
@@ -156,9 +191,29 @@ handle_key(uim_context uc, char *p, int key, int state)
   uim_eval_string(uc, buf);
 }
 
+static int
+emergency_key_p(int key, int state)
+{
+  if ((state == UMod_Control) &&
+      (key == UKey_Shift_key)) {
+    return 1;
+  }
+  return 0;
+}
+
 int
 uim_press_key(uim_context uc, int key, int state)
 {
+  if (!uc) {
+    return 1;
+  }
+  if (emergency_key_p(key, state)) {
+    uc->is_enable = uc->is_enable ? 0 : 1;
+    return 0;
+  }
+  if (!uc->is_enable) {
+    return 1;
+  }
   uc->commit_raw_flag = 0;
   handle_key(uc, "press", key, state);
   return uc->commit_raw_flag;
@@ -167,6 +222,12 @@ uim_press_key(uim_context uc, int key, int state)
 int
 uim_release_key(uim_context uc, int key, int state)
 {
+  if (!uc) {
+    return 1;
+  }
+  if (!uc->is_enable) {
+    return 1;
+  }
   uc->commit_raw_flag = 0;
   handle_key(uc, "release", key, state);
   return uc->commit_raw_flag;
