@@ -1,3 +1,38 @@
+/*
+
+  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+
+  All rights reserved.
+
+  Redistribution and use in source and binary forms, with or without
+  modification, are permitted provided that the following conditions
+  are met:
+
+  1. Redistributions of source code must retain the above copyright
+     notice, this list of conditions and the following disclaimer.
+  2. Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+  3. Neither the name of authors nor the names of its contributors
+     may be used to endorse or promote products derived from this software
+     without specific prior written permission.
+
+  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+  SUCH DAMAGE.
+
+*/
+
+#include <stdlib.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
@@ -16,7 +51,7 @@ struct table_info {
   char *filename;
 };
 
-static struct table_info *
+static int
 open_table_internal(const char *fn)
 {
   struct table_info *table;
@@ -26,18 +61,18 @@ open_table_internal(const char *fn)
   void *addr;
 
   if (lstat(fn, &st) == -1) {
-    return NULL;
+    return 0;
   }
 
   fd = open(fn, O_RDONLY);
   if (fd == -1) {
-    return NULL;
+    return 0;
   }
 
   addr = mmap(0, st.st_size, PROT_READ, MAP_SHARED, fd, 0);
   close(fd);
   if (addr == MAP_FAILED) {
-    return NULL;
+    return 0;
   }
   table = (struct table_info *)malloc(sizeof(struct table_info));
   table->addr = addr;
@@ -50,7 +85,7 @@ open_table_internal(const char *fn)
       return i;
     }
   }
-  return NULL;
+  return 0;
 }
 
 static void
