@@ -1,3 +1,36 @@
+/*
+
+  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+
+  All rights reserved.
+
+  Redistribution and use in source and binary forms, with or without
+  modification, are permitted provided that the following conditions
+  are met:
+
+  1. Redistributions of source code must retain the above copyright
+     notice, this list of conditions and the following disclaimer.
+  2. Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+  3. Neither the name of authors nor the names of its contributors
+     may be used to endorse or promote products derived from this software
+     without specific prior written permission.
+
+  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+  SUCH DAMAGE.
+
+*/
+
 #include <pwd.h>
 #include <unistd.h>
 #include <sys/types.h>
@@ -24,6 +57,8 @@ extern void uim_init_anthy();
 extern void uim_quit_anthy();
 extern void uim_init_prime();
 extern void uim_init_canna();
+extern void uim_init_m17nlib();
+extern void uim_quit_m17nlib();
 
 #define CONTEXT_ARRAY_SIZE 512
 static uim_context context_array[CONTEXT_ARRAY_SIZE];
@@ -76,6 +111,10 @@ uim_create_context(void *ptr,
   int buf_len;
   char *buf;
 
+  if (!conv) {
+    conv = uim_iconv;
+  }
+
   if (uim_siod_fatal || !conv) {
     return NULL;
   }
@@ -86,6 +125,7 @@ uim_create_context(void *ptr,
   }
   get_context_id(uc);
   uc->ptr = ptr;
+  uc->is_enable = 1;
   uc->commit_cb = commit_cb;
   if (!enc) {
     enc = "UTF-8";
@@ -112,10 +152,10 @@ uim_create_context(void *ptr,
   uc->prop_list_update_cb  = NULL;
   uc->prop_label_update_cb = NULL;
   /**/
-  uc->candidate_selector_begin_cb = NULL;
+  uc->candidate_selector_activate_cb = NULL;
   uc->candidate_selector_select_cb = NULL;
   uc->candidate_selector_shift_page_cb = NULL;
-  uc->candidate_selector_end_cb = NULL;
+  uc->candidate_selector_deactivate_cb = NULL;
   /**/
   uc->nr_candidates = 0;
   uc->candidate_index = 0;
@@ -217,7 +257,7 @@ uim_set_prop_label_update_cb(uim_context uc,
 
 
 void
-uim_prop_activate(uim_context uc, char *str)
+uim_prop_activate(uim_context uc, const char *str)
 {
   char buf[100];
   
@@ -253,13 +293,15 @@ uim_set_mode_cb(uim_context uc, void (*update_cb)(void *ptr,
 void
 uim_prop_list_update(uim_context uc)
 {
-  uc->prop_list_update_cb(uc, uc->propstr);
+  if(uc && uc->propstr)
+    uc->prop_list_update_cb(uc, uc->propstr);
 }
 
 void
 uim_prop_label_update(uim_context uc)
 {
-  uc->prop_label_update_cb(uc, uc->proplabelstr);
+  if(uc && uc->proplabelstr)
+    uc->prop_label_update_cb(uc, uc->proplabelstr);
 }
 
 int
@@ -352,14 +394,14 @@ load_conf()
 }
 
 int uim_set_candidate_selector_cb(uim_context uc,
-				  void (*begin_cb)(void *ptr, int nr, int display_limit),
+				  void (*activate_cb)(void *ptr, int nr, int display_limit),
 				  void (*select_cb)(void *ptr, int index),
 				  void (*shift_page_cb)(void *ptr, int direction),
-				  void (*end_cb)(void *ptr))
+				  void (*deactivate_cb)(void *ptr))
 {
-  uc->candidate_selector_begin_cb = begin_cb;
+  uc->candidate_selector_activate_cb = activate_cb;
   uc->candidate_selector_select_cb = select_cb;
-  uc->candidate_selector_end_cb = end_cb;
+  uc->candidate_selector_deactivate_cb = deactivate_cb;
   uc->candidate_selector_shift_page_cb = shift_page_cb;
 
   return 0;
@@ -428,6 +470,14 @@ exit_hook(void)
   uim_siod_fatal = 1;
 }
 
+static void
+load_file(char *fn)
+{
+  char *ln = alloca(strlen(fn)+40);
+  sprintf(ln, "(*catch 'errobj (load \"%s\" #f #f))", fn);
+  repl_c_string(ln, 0, 0);
+}
+
 static void
 uim_init_scm()
 {
@@ -436,7 +486,7 @@ uim_init_scm()
   char *siod_argv[] =
     {
       "siod",
-      "-h300000:10",
+      "-h100000:10",
       "-o1000",
       "-s200000",
       "-n128",
@@ -468,6 +518,9 @@ uim_init_scm()
 #ifdef HAVE_CANNA_RK_H
   uim_init_canna();
 #endif /* HAVE_CANNA_RK_H */
+#ifdef HAVE_M17NLIB
+  uim_init_m17nlib();
+#endif
 
   scm_files = getenv("LIBUIM_SCM_FILES");
   if (scm_files) {
@@ -476,11 +529,11 @@ uim_init_scm()
     siod_lib = SCM_FILES;
   }
 
-  vload("im.scm" , 0, 1);
-  vload("loader.scm" , 0, 1);
+  load_file("im.scm");
+  load_file("loader.scm");
   if (getenv("LIBUIM_VANILLA") ||
       load_conf() == -1) {
-    vload("default.scm", 0, 1);
+    load_file("default.scm");
   }
   if (getenv("LIBUIM_VERBOSE")) {
     siod_verbose_level = 10;
@@ -522,6 +575,9 @@ uim_quit(void)
 #ifdef HAVE_CANNA_RK_H
   uim_quit_canna();
 #endif /* HAVE_CANNA_RK_H */
+#ifdef HAVE_M17NLIB
+  uim_quit_m17nlib();
+#endif
   siod_quit();
   uim_initialized = 0;
 }
