@@ -1,3 +1,36 @@
+/*
+
+  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+
+  All rights reserved.
+
+  Redistribution and use in source and binary forms, with or without
+  modification, are permitted provided that the following conditions
+  are met:
+
+  1. Redistributions of source code must retain the above copyright
+     notice, this list of conditions and the following disclaimer.
+  2. Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+  3. Neither the name of authors nor the names of its contributors
+     may be used to endorse or promote products derived from this software
+     without specific prior written permission.
+
+  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+  SUCH DAMAGE.
+
+*/
+
 /* API and ABI is unstable */
 #ifndef _uim_h_included_
 #define _uim_h_included_
@@ -109,28 +142,54 @@ enum UKey {
   UKey_Private29,
   UKey_Private30,
   /* this part is especially unstable */
+  /* modifier keys: "_key" suffix should be removed in the future */
   UKey_Shift_key,
   UKey_Control_key,
   UKey_Alt_key,
   UKey_Meta_key,
+  UKey_Super_key,
+  UKey_Hyper_key,
   UKey_Other = 1000
 };
   
 enum UKeyModifier {
-  UKey_Shift = 1,
-  UKey_Control = 2,
-  UKey_Alt = 4,
-  UKey_Meta = 8,
-  UKey_Pseudo0 = 16,
-  UKey_Pseudo1 = 32,
+  UMod_Shift = 1,
+  UMod_Control = 2,
+  UMod_Alt = 4,
+  UMod_Meta = 8,
+  UMod_Pseudo0 = 16,
+  UMod_Pseudo1 = 32,
+  UMod_Super = 64,
+  UMod_Hyper = 128
+
+#ifndef UIM_NO_BACKWARD_COMPATIBLE_API
+  /* will be deprecated */
+  ,
+  UKey_Shift = UMod_Shift,
+  UKey_Control = UMod_Control,
+  UKey_Alt = UMod_Alt,
+  UKey_Meta = UMod_Meta,
+  UKey_Pseudo0 = UMod_Pseudo0,
+  UKey_Pseudo1 = UMod_Pseudo1
+#endif
 };
   
 enum UPreeditAttr {
-  UPeAttr_None = 0,
-  UPeAttr_UnderLine = 1,
-  UPeAttr_Reverse = 2,
-  UPeAttr_Cursor = 4,
-  UPeAttr_Separator = 8,
+  UPreeditAttr_None = 0,
+  UPreeditAttr_UnderLine = 1,
+  UPreeditAttr_Reverse = 2,
+  UPreeditAttr_Cursor = 4,
+  UPreeditAttr_Separator = 8
+
+#ifndef UIM_NO_BACKWARD_COMPATIBLE_API
+  /* will be deprecated */
+  ,
+  UPeAttr_None = UPreeditAttr_None,
+  UPeAttr_UnderLine = UPreeditAttr_UnderLine,
+  UPeAttr_Reverse = UPreeditAttr_Reverse,
+  UPeAttr_Cursor = UPreeditAttr_Cursor,
+  UPeAttr_Separator = UPreeditAttr_Separator
+#endif
 };
 
 /* abstracting platform-dependent character code conversion method */
@@ -199,7 +258,7 @@ void
 uim_set_prop_label_update_cb(uim_context uc,
 			     void (*update_cb)(void *ptr, char *str));
 void
-uim_prop_activate(uim_context uc, char *str);
+uim_prop_activate(uim_context uc, const char *str);
 /* mode */
 int
 uim_get_current_mode(uim_context uc);
@@ -210,10 +269,10 @@ uim_set_mode_cb(uim_context uc, void (*update_cb)(void *ptr,
 						  int mode));
   
 int uim_set_candidate_selector_cb(uim_context uc,
-				  void (*begin_cb)(void *ptr, int nr, int display_limit),
+				  void (*activate_cb)(void *ptr, int nr, int display_limit),
 				  void (*select_cb)(void *ptr, int index),
 				  void (*shift_page_cb)(void *ptr, int direction),
-				  void (*end_cb)(void *ptr));
+				  void (*deactivate_cb)(void *ptr));
 
 /* You must free the result by uim_candidate_free */
 uim_candidate uim_get_candidate(uim_context uc, int index, int accel_enumeration_hint);
