@@ -1,3 +1,35 @@
+/*
+
+  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+
+  All rights reserved.
+
+  Redistribution and use in source and binary forms, with or without
+  modification, are permitted provided that the following conditions
+  are met:
+
+  1. Redistributions of source code must retain the above copyright
+     notice, this list of conditions and the following disclaimer.
+  2. Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+  3. Neither the name of authors nor the names of its contributors
+     may be used to endorse or promote products derived from this software
+     without specific prior written permission.
+
+  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+  SUCH DAMAGE.
+*/
+
 // XIM Server currently for Japanese
 // initialize many modules
 //
@@ -27,7 +59,6 @@ int scr_width,scr_height;
 int host_byte_order;
 
 int lib_uim_fd;
-static char *result;
 
 #define VERSION_NAME "uim-xim under the way! Version "PACKAGE_VERSION"\n"
 char *version_name=VERSION_NAME;
@@ -354,7 +385,30 @@ get_runtime_env()
 static void
 helper_str_parse(char *str)
 {
-    //currently do nothing.
+    InputContext *focusedContext = InputContext::focusedContext();
+    
+    if (focusedContext) {
+	char *line = str;	
+	char *eol = strchr(line, '\n');
+	if (eol != NULL) {
+	    *eol = '\0';
+	}
+
+	if (strcmp("prop_list_get", line) == 0) {
+	    uim_prop_list_update(focusedContext->getUC());
+	} else if (strcmp("prop_label_get", line) == 0) {
+	    uim_prop_label_update(focusedContext->getUC());
+	} else if (strcmp("prop_activate", line) == 0) {
+	    line = eol + 1;
+	    
+	    eol = strchr(line, '\n');
+	    if (eol != NULL) {
+		*eol = '\0';
+	    }
+
+	    uim_prop_activate(focusedContext->getUC(), line);
+	}
+    }
 }
 
 static void
