@@ -1,3 +1,35 @@
+/*
+
+  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+
+  All rights reserved.
+
+  Redistribution and use in source and binary forms, with or without
+  modification, are permitted provided that the following conditions
+  are met:
+
+  1. Redistributions of source code must retain the above copyright
+     notice, this list of conditions and the following disclaimer.
+  2. Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+  3. Neither the name of authors nor the names of its contributors
+     may be used to endorse or promote products derived from this software
+     without specific prior written permission.
+
+  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+  SUCH DAMAGE.
+*/
+
 // XIM ץȥIMνԤ
 
 #ifdef HAVE_CONFIG_H
@@ -247,31 +279,6 @@ void XimIM_impl::forward_event(RxPacket *p)
 	k.press = (k.ev.type == KeyPress);
 	k.key_sym = ks;
 
-	switch (ks) {
-	case XK_KP_0:
-	case XK_KP_1:
-	case XK_KP_2:
-	case XK_KP_3:
-	case XK_KP_4:
-	case XK_KP_5:
-	case XK_KP_6:
-	case XK_KP_7:
-	case XK_KP_8:
-	case XK_KP_9:
-	case XK_KP_Equal:
-	case XK_KP_Multiply:
-	case XK_KP_Add:
-	case XK_KP_Separator:
-	case XK_KP_Subtract:
-	case XK_KP_Decimal:
-	case XK_KP_Divide:
-	    k.key_sym = ks - XK_KP_Space;
-	    break;
-	case XK_KP_Enter:
-	    k.key_sym = XK_Return;
-	    break;
-	}
-
 	if (ic) {
 	    ic->OnKeyEvent(k);
 	}
