@@ -1,3 +1,35 @@
+/*
+
+  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+
+  All rights reserved.
+
+  Redistribution and use in source and binary forms, with or without
+  modification, are permitted provided that the following conditions
+  are met:
+
+  1. Redistributions of source code must retain the above copyright
+     notice, this list of conditions and the following disclaimer.
+  2. Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+  3. Neither the name of authors nor the names of its contributors
+     may be used to endorse or promote products derived from this software
+     without specific prior written permission.
+
+  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+  SUCH DAMAGE.
+*/
+
 #include <ctype.h>
 #include <X11/Xatom.h>
 #include "xim.h"
@@ -14,6 +46,7 @@
 
 extern int lib_uim_fd;
 extern Atom xim_servers;
+InputContext *InputContext::mFocusedContext = NULL;
 
 // tables
 static input_style input_style_tab_with_over_the_spot[] = {
@@ -107,27 +140,7 @@ XimServer::XimServer(Locale *lc, char *name, char *lang)
 
 InputContext *XimServer::createContext(XimIC *xic)
 {
-    InputContext *ic = new InputContext(this, xic);
-    uim_context uc = uim_create_context((void *) ic, "EUC-JP",
-					NULL, mName, uim_iconv,
-					InputContext::commit_cb);
-    uim_set_preedit_cb(uc,
-		       InputContext::clear_cb,
-		       InputContext::pushback_cb,
-		       InputContext::update_cb);
-    uim_set_candidate_selector_cb(uc,
-				  InputContext::candidate_begin_cb,
-				  InputContext::candidate_select_cb,
-				  NULL,
-				  InputContext::candidate_end_cb);
-    uim_set_prop_list_update_cb(uc,
-				InputContext::update_prop_list_cb);
-    uim_set_prop_label_update_cb(uc,
-				 InputContext::update_prop_label_cb);
-
-    uim_prop_list_update(uc);
-    
-    ic->setUC(uc);
+    InputContext *ic = new InputContext(this, xic, mName);
     return ic;
 }
 
@@ -240,17 +253,20 @@ XimServer *XimServer::findServer(Window w)
 
 //
 // InputContextΥ᥽å
-InputContext::InputContext(XimServer *svr, XimIC *ic)
+InputContext::InputContext(XimServer *svr, XimIC *ic, char *engine)
 {
     mXic = ic;
     m_pe = new pe_stat(this);
     mConvdisp = 0;
-    mUc = 0;
+    mUc = createUimContext(engine);
     mServer = svr;
 }
 
 InputContext::~InputContext()
 {
+    if (mFocusedContext == this) {
+	mFocusedContext = NULL;
+    }
     if (mConvdisp) {
 	mConvdisp->set_pe(0);
     }
@@ -258,11 +274,45 @@ InputContext::~InputContext()
     uim_release_context(mUc);
 }
 
+uim_context
+InputContext::createUimContext(char *engine)
+{
+    uim_context uc = uim_create_context((void *) this, "EUC-JP",
+					NULL, engine, uim_iconv,
+					InputContext::commit_cb);
+
+    uim_set_preedit_cb(uc,
+		       InputContext::clear_cb,
+		       InputContext::pushback_cb,
+		       InputContext::update_cb);
+    uim_set_candidate_selector_cb(uc,
+				  InputContext::candidate_activate_cb,
+				  InputContext::candidate_select_cb,
+				  NULL,
+				  InputContext::candidate_deactivate_cb);
+    uim_set_prop_list_update_cb(uc,
+				InputContext::update_prop_list_cb);
+    uim_set_prop_label_update_cb(uc,
+				 InputContext::update_prop_label_cb);
+
+    uim_prop_list_update(uc);
+
+    return uc;
+}
+
+InputContext*
+InputContext::focusedContext()
+{
+    return mFocusedContext;
+}
+
 void
 InputContext::focusIn()
 {
     uim_prop_list_update(mUc);	
+    uim_prop_label_update(mUc);	
     uim_helper_client_focus_in(mUc);
+    mFocusedContext = this;
 }
 
 void
@@ -277,10 +327,10 @@ InputContext::getServer()
     return mServer;
 }
 
-void
-InputContext::setUC(uim_context uc)
+uim_context
+InputContext::getUC()
 {
-    mUc = uc;
+    return mUc;
 }
 
 void
@@ -312,10 +362,10 @@ void InputContext::update_cb(void *ptr)
     ic->update_preedit();
 }
 
-void InputContext::candidate_begin_cb(void *ptr, int nr, int display_limit)
+void InputContext::candidate_activate_cb(void *ptr, int nr, int display_limit)
 {
     InputContext *ic = (InputContext *)ptr;
-    ic->candidate_begin(nr, display_limit);
+    ic->candidate_activate(nr, display_limit);
 }
 
 void InputContext::candidate_select_cb(void *ptr, int index)
@@ -324,10 +374,10 @@ void InputContext::candidate_select_cb(void *ptr, int index)
     ic->candidate_select(index);
 }
 
-void InputContext::candidate_end_cb(void *ptr)
+void InputContext::candidate_deactivate_cb(void *ptr)
 {
     InputContext *ic = (InputContext *)ptr;
-    ic->candidate_end();
+    ic->candidate_deactivate();
 }
 
 void InputContext::update_prop_list_cb(void *ptr, char *str)
@@ -350,13 +400,13 @@ void InputContext::clear_preedit()
 void InputContext::pushback_preedit_string(int attr, char *str)
 {
     int p = 0;
-    if (attr & UPeAttr_UnderLine) {
+    if (attr & UPreeditAttr_UnderLine) {
 	p |= PE_UNDERLINE;
     }
-    if (attr & UPeAttr_Reverse) {
+    if (attr & UPreeditAttr_Reverse) {
 	p |= PE_REVERSE;
     }
-    if (attr & UPeAttr_Cursor) {
+    if (attr & UPreeditAttr_Cursor) {
 	m_pe->caret_pos = m_pe->get_char_count();
     }
     if (!strlen(str)) {
@@ -391,7 +441,7 @@ int InputContext::pushKey(keyState *k)
 	    return UPDATE_MODE;
 	}
     }
-    return DO_NOTHING;
+    return COMMIT_RAW;
 }
 
 int InputContext::getMode()
@@ -430,7 +480,7 @@ XimIC *InputContext::get_ic()
     return mXic;
 }
 
-void InputContext::candidate_begin(int nr, int display_limit)
+void InputContext::candidate_activate(int nr, int display_limit)
 {
     int i;
     std::vector<const char*> candidates;
@@ -442,7 +492,7 @@ void InputContext::candidate_begin(int nr, int display_limit)
 	cand = uim_get_candidate(mUc, i,i);
         candidates.push_back(uim_candidate_get_cand_str(cand));
     }
-    disp->begin(candidates);
+    disp->activate(candidates);
     //    disp->update();
 }
 
@@ -452,10 +502,10 @@ void InputContext::candidate_select(int index)
     disp->select(index);
 }
 
-void InputContext::candidate_end()
+void InputContext::candidate_deactivate()
 {
     Canddisp* disp = canddisp_singleton();
-    disp->end();
+    disp->deactivate();
 }
 
 void InputContext::update_prop_list(char* str)
@@ -488,16 +538,22 @@ keyState::keyState(keyEventX *x)
 {
     mModifier = 0;
     if (x->state & 1) {
-	mModifier += UKey_Shift;
+	mModifier += UMod_Shift;
     }
     if (x->state & 2) {
-	mModifier |= UKey_Alt;
+	mModifier |= UMod_Alt;
     }
     if (x->state & 4) {
-	mModifier |= UKey_Control;
+	mModifier |= UMod_Control;
     }
     if (x->state & 8) {
-	mModifier |= UKey_Meta;
+	mModifier |= UMod_Meta;
+    }
+    if (x->state & 32) {  /* assuming mod3 */
+	mModifier |= UMod_Super;
+    }
+    if (x->state & 64) {  /* assuming mod4 */
+	mModifier |= UMod_Hyper;
     }
     if (x->key_sym < 128 && x->key_sym >= 32) {
 	mKey = x->key_sym;
@@ -533,6 +589,37 @@ keyState::keyState(keyEventX *x)
 	case XK_Mode_switch: mKey = UKey_Mode_switch; break;
 	case XK_Henkan_Mode: mKey = UKey_Henkan_Mode; break;
 	case XK_Muhenkan: mKey = UKey_Muhenkan; break;
+	case XK_KP_0:
+	case XK_KP_1:
+	case XK_KP_2:
+	case XK_KP_3:
+	case XK_KP_4:
+	case XK_KP_5:
+	case XK_KP_6:
+	case XK_KP_7:
+	case XK_KP_8:
+	case XK_KP_9:
+	case XK_KP_Equal:
+	case XK_KP_Multiply:
+	case XK_KP_Add:
+	case XK_KP_Separator:
+	case XK_KP_Subtract:
+	case XK_KP_Decimal:
+	case XK_KP_Divide:
+	case XK_KP_Enter:
+	case XK_KP_Home:
+	case XK_KP_Left:
+	case XK_KP_Up:
+	case XK_KP_Right:
+	case XK_KP_Down:
+	case XK_KP_Prior:
+	case XK_KP_Next:
+	case XK_KP_End:
+	case XK_KP_Begin:
+	case XK_KP_Insert:
+	case XK_KP_Delete:
+	    mKey = x->key_sym;
+	    break;
 	default: mKey = UKey_Other;
 	}
     }
