@@ -1,3 +1,35 @@
+/*
+
+  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+
+  All rights reserved.
+
+  Redistribution and use in source and binary forms, with or without
+  modification, are permitted provided that the following conditions
+  are met:
+
+  1. Redistributions of source code must retain the above copyright
+     notice, this list of conditions and the following disclaimer.
+  2. Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+  3. Neither the name of authors nor the names of its contributors
+     may be used to endorse or promote products derived from this software
+     without specific prior written permission.
+
+  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+  SUCH DAMAGE.
+*/
+
 // -*- C++ -*-
 #ifndef _ximserver_h_included_
 #define _ximserver_h_included_
@@ -109,9 +141,9 @@ class Convdisp;
 class XimServer;
 class InputContext {
 public:
-    InputContext(XimServer *, XimIC *);
+    InputContext(XimServer *, XimIC *, char *);
     ~InputContext();
-    void setUC(uim_context );
+    uim_context getUC();
     int pushKey(keyState *e);
     int getMode();
     void setMode(int mode);
@@ -125,9 +157,9 @@ public:
     void clear_preedit();
     void pushback_preedit_string(int attr, char *str);
     void update_preedit();
-    void candidate_begin(int nr, int display_limit);
+    void candidate_activate(int nr, int display_limit);
     void candidate_select(int index);
-    void candidate_end();
+    void candidate_deactivate();
     void update_prop_list(char *str);
     void update_prop_label(char *str);
 public:
@@ -135,11 +167,12 @@ public:
     static void clear_cb(void *);
     static void pushback_cb(void *, int attr, char *str);
     static void update_cb(void *);
-    static void candidate_begin_cb(void *ptr, int nr, int index);
+    static void candidate_activate_cb(void *ptr, int nr, int index);
     static void candidate_select_cb(void *ptr, int index);
-    static void candidate_end_cb(void *ptr);
+    static void candidate_deactivate_cb(void *ptr);
     static void update_prop_list_cb(void *ptr, char *str);
     static void update_prop_label_cb(void *ptr, char *str);
+    static InputContext *focusedContext();
 protected:
     void commit_string(char *);
     XimIC *mXic;
@@ -147,6 +180,9 @@ protected:
     pe_stat *m_pe;
     Convdisp *mConvdisp;
     uim_context mUc;
+private:
+    uim_context createUimContext(char *engine);
+    static InputContext *mFocusedContext;    
 };
 
 class Locale {
